﻿using System;
using System.ComponentModel;

namespace Chocoat
{
    /// <summary>
    /// CCAPIの呼び出しをコンポーネント化し、WinFormに貼り付けて利用可能にしたクラスです。
    /// </summary>
    /// <remarks>
    /// <para>
    /// 通常はこのコンポーネントを利用して、CHOCOAと対話してください。
    /// </para>
    /// </remarks>
    [DefaultEvent("MessageEvent")]
    [DefaultProperty("Enable")]
    public sealed class ChocoatComponents : Component, ISupportInitialize, IChocoat
    {
        private Core _chocoatCore;

        #region constructors
        /// <summary>
        /// ChocoatComponents の新しいインスタンスを生成します。
        /// </summary>
        public ChocoatComponents()
        {
            InitializeComponent();
            this._enable = true;
        }

        /// <summary>
        /// ChocoatComponents の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="container">コンテナ</param>
        public ChocoatComponents(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
        }

        /// <summary>
        /// 初期化の部分
        /// </summary>
        /// <remarks>
        /// <para>
        /// ここが原因で、コンポーネントがデザイナで開けないとか言われるかもしれないが、これ、右クリックで「コードの表示」以外でメンテしない。
        /// </para>
        /// </remarks>
        private void InitializeComponent()
        {
            if (!this.DesignMode)
            {
                this._chocoatCore = new Core();

                //イベント
                this.AddChocoaEvents();

                //プロパティ
                this.AddChocoaPropertiesEvents();
            }
        }

        #region AddChocoaEvents / CHOCOAイベントの追加
        /// <summary>
        /// CHOCOAイベントの追加
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1506:AvoidExcessiveClassCoupling")]
        private void AddChocoaEvents()
        {
            this._chocoatCore.MessageEvent += delegate(object sender, MessageEventArgs e) { this.OnMessageEvent(e); };

            this._chocoatCore.IRCNickEvent += delegate(object sender, IRCNickEventArgs e) { this.OnIRCNickEvent(e); };
            this._chocoatCore.IRCJoinEvent += delegate(object sender, IRCJoinEventArgs e) { this.OnIRCJoinEvent(e); };
            this._chocoatCore.IRCTopicEvent += delegate(object sender, IRCTopicEventArgs e) { this.OnIRCTopicEvent(e); };
            this._chocoatCore.IRCNamesEvent += delegate(object sender, IRCNamesEventArgs e) { this.OnIRCNamesEvent(e); };
            this._chocoatCore.IRCPartEvent += delegate(object sender, IRCPartEventArgs e) { this.OnIRCPartEvent(e); };
            this._chocoatCore.IRCQuitEvent += delegate(object sender, IRCQuitEventArgs e) { this.OnIRCQuitEvent(e); };
            this._chocoatCore.IRCModeEvent += delegate(object sender, IRCModeEventArgs e) { this.OnIRCModeEvent(e); };
            this._chocoatCore.IRCKickEvent += delegate(object sender, IRCKickEventArgs e) { this.OnIRCKickEvent(e); };
            this._chocoatCore.IRCInviteEvent += delegate(object sender, IRCInviteEventArgs e) { this.OnIRCInviteEvent(e); };
            this._chocoatCore.IRCActionEvent += delegate(object sender, IRCActionEventArgs e) { this.OnIRCActionEvent(e); };
            this._chocoatCore.IRCIsonEvent += delegate(object sender, IRCIsonEventArgs e) { this.OnIRCIsonEvent(e); };
            this._chocoatCore.IRCWhoisEvent += delegate(object sender, IRCWhoisEventArgs e) { this.OnIRCWhoisEvent(e); };
            this._chocoatCore.IRCWhoEvent += delegate(object sender, IRCWhoEventArgs e) { this.OnIRCWhoEvent(e); };

            this._chocoatCore.ScriptEvent += delegate(object sender, ScriptEventArgs e) { this.OnScriptEvent(e); };
            this._chocoatCore.NumericMessageEvent += delegate(object sender, NumericMessageEventArgs e) { this.OnNumericMessageEvent(e); };
            this._chocoatCore.ChocoaWindowEvent += delegate(object sender, ChocoaWindowEventArgs e) { this.OnChocoaWindowEvent(e); };
        }
        #endregion

        #region AddChocoaPropertiesEvents / CHOCOAのプロパティ変更に関するイベントの追加
        /// <summary>
        /// CHOCOAのプロパティ変更に関するイベントの追加
        /// </summary>
        private void AddChocoaPropertiesEvents()
        {
            this._chocoatCore.ChangeSupportMessageEvent += delegate(object sender, SupportPropertyChangedEventArgs e)
            {
                this._supportMessageEvent = e.Value;
            };

            this._chocoatCore.ChangeSupportCTCPMessageEvent += delegate(object sender, SupportPropertyChangedEventArgs e)
            {
                this._supportCTCPMessageEvent = e.Value;
            };

            this._chocoatCore.ChangeSupportIRCEvent += delegate(object sender, SupportPropertyChangedEventArgs e)
            {
                this._supportIRCEvent = e.Value;
            };

            this._chocoatCore.ChangeSupportScriptEvent += delegate(object sender, SupportPropertyChangedEventArgs e)
            {
                this._supportScriptEvent = e.Value;
            };

            this._chocoatCore.ChangeSupportNumericMessageEvent += delegate(object sender, SupportPropertyChangedEventArgs e)
            {
                this._supportNumericMessageEvent = e.Value;
            };

            this._chocoatCore.ChangeSupportChocoaEvent += delegate(object sender, SupportPropertyChangedEventArgs e)
            {
                this._supportChocoaEvent = e.Value;
            };
        }
        #endregion

        #endregion

        #region Dispose
        /// <summary>
        /// disposeが呼び出されたか
        /// </summary>
        private bool _disposed;

        /// <summary>
        /// Component によって使用されているアンマネージ リソースを解放し、オプションでマネージ リソースも解放します。
        /// </summary>
        /// <param name="disposing">マネージ リソースとアンマネージ リソースの両方を解放する場合は true。アンマネージ リソースだけを解放する場合は false。</param>
        protected override void Dispose(bool disposing)
        {
            if (!this._disposed)
            {
                if (disposing)
                {
                    this._chocoatCore.Dispose();
                    this._chocoatCore = null;
                }
            }

            this._disposed = true;

            base.Dispose(disposing);
        }
        #endregion

        #region Enable / CCAPIを有効にするかを取得、または設定します。

        private bool _enable;

        /// <summary>
        /// CCAPIを有効にするかを取得、または設定します。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CCAPIを有効にするかを取得、または設定します。")]
        public bool Enable
        {
            get
            {
                return this._enable;
            }
            set
            {
                this._enable = value;

                if (this._enable)
                {
                    this.Start();
                }
                else
                {
                    this.Stop();
                }
            }
        } 
        #endregion

        #region ISupportInitialize メンバ

        private bool _initializing;

        /// <summary>
        /// 初期化の開始を通知するシグナルをオブジェクトに送信します。 
        /// </summary>
        void ISupportInitialize.BeginInit()
        {
            this._initializing = true;
        }

        /// <summary>
        /// 初期化の完了を通知するシグナルをオブジェクトに送信します。
        /// </summary>
        void ISupportInitialize.EndInit()
        {
            this._initializing = false;

            if (this.Enable)
            {
                this.Start();
            }
        }
        #endregion

        #region Start / CCAPIの利用を開始します。
        /// <summary>
        /// CCAPIの利用を開始します。
        /// </summary>
        /// <returns>正常に開始出来たら、<c>true</c> 。</returns>
        public bool Start()
        {
            //初期化が終わるまではやんない
            if (!this._initializing && !this.DesignMode)
            {
                if (!this._chocoatCore.IsStart)
                {
                    this._enable = this._chocoatCore.Start();

                    if (this._enable)
                    {
                        this.SetEvents();
                    }
                }
            }

            return this.Enable;
        } 
        #endregion

        #region Stop / CCAPIを停止します。
        /// <summary>
        /// CCAPIを停止します。
        /// </summary>
        public void Stop()
        {
            this._chocoatCore.Stop();
        } 
        #endregion

        //ちょこあさん
        #region HasChocoa / CHOCOAクライアントのCCAPIが開始されているかを取得します。
        /// <summary>
        /// CHOCOAクライアントのCCAPIが開始されているかを取得します。
        /// </summary>
        [Browsable(false)]
        public bool IsStart
        {
            get
            {
                return this._chocoatCore.IsStart;
            }
        }
        #endregion

        #region Version / インストールされているCHOCOAのバージョンを取得します。インストールされていない時は <c>cull</c> です。
        /// <summary>
        /// インストールされているCHOCOAのバージョンを取得します。インストールされていない時は <c>null</c> です。
        /// </summary>
        [Browsable(false)]
        public string Version
        {
            get
            {
                return this._chocoatCore.Version;
            }
        }
        #endregion

        #region SettingServerCount / 設定されているサーバーの個数を取得します。
        /// <summary>
        /// 設定されているサーバーの個数を取得します。
        /// </summary>
        /// <returns>設定されているサーバーの個数</returns>
        [Browsable(false)]
        public int SettingServerCount
        {
            get
            {
                return this._chocoatCore.SettingServerCount;
            }
        }
        #endregion

        #region HideWindow / ウィンドウを非表示にします。
        /// <summary>
        /// ウィンドウを非表示にします。
        /// </summary>
        public void HideWindow()
        {
            this._chocoatCore.HideWindow();
        }

        /// <summary>
        /// 指定されたサーバーIDと接続されているウィンドウを非表示にします。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        public void HideWindow(int serverID)
        {
            this._chocoatCore.HideWindow(serverID);
        }
        #endregion

        #region ShowWindow / ウィンドウを表示します。
        /// <summary>
        /// ウィンドウを表示します。
        /// </summary>
        public void ShowWindow()
        {
            this._chocoatCore.ShowWindow();
        }

        /// <summary>
        /// 指定されたサーバーIDに接続されているウィンドウを表示します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        public void ShowWindow(int serverID)
        {
            this._chocoatCore.ShowWindow(serverID);
        }
        #endregion

        #region IsShowWindow / 指定されたサーバーと接続しているCHOCOAウィンドウが、表示状態にあるかをチェックします。
        /// <summary>
        /// 指定されたサーバーと接続しているCHOCOAウィンドウが、表示状態にあるかをチェックします。
        /// </summary>
        /// <param name="serverID">接続を確認するサーバーID。0を指定すると、すべてのウィンドウをチェックします。</param>
        /// <returns>表示され、接続されている時は <c>true</c> 。それ以外は <c>fakse</c> 。</returns>
        public bool IsShowWindow(int serverID)
        {
            return this._chocoatCore.IsShowWindow(serverID);
        } 
        #endregion

        #region GetSettingServerName / 指定されたサーバーIDの設定名を取得します。
        /// <summary>
        /// 指定されたサーバーIDの設定名を取得します。
        /// </summary>
        /// <param name="serverID">取得するサーバーID</param>
        /// <returns>サーバー設定名。存在しない場合は <c>null</c> 。</returns>
        public string GetSettingServerName(int serverID)
        {
            return this._chocoatCore.GetSettingServerName(serverID);
        }
        #endregion

        #region GetSettingServerNames / 設定されているサーバー名を取得します。
        /// <summary>
        /// 設定されているサーバー名を取得します。
        /// </summary>
        /// <returns>取得したサーバー名の配列。</returns>
        public string[] GetSettingServerNames()
        {
            return this._chocoatCore.GetSettingServerNames();
        }
        #endregion

        #region FindServerID / CHOCOAから、サーバーの設定名を元に、CHOCOA内の管理IDを取得します。
        /// <summary>
        /// CHOCOAから、サーバーの設定名を元に、CHOCOA内の管理IDを取得します。
        /// </summary>
        /// <param name="settingName">CHOCOAのサーバー設定名。</param>
        /// <returns>取得した内部管理番号。正常に終了したら 0 以上のID。失敗時は 0 。</returns>
        public int FindServerID(string settingName)
        {
            return this._chocoatCore.FindServerID(settingName);
        }
        #endregion

        #region IsConnect / 指定されたサーバーIDが接続状態にあるかを取得します。
        /// <summary>
        /// 指定されたサーバーIDが接続状態にあるかを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <returns>接続されていれば <c>true</c> 。接続されていない時は <c>false</c> 。</returns>
        public bool IsConnect(int serverID)
        {
            return this._chocoatCore.IsConnect(serverID);
        }
        #endregion

        #region CheckScriptServiceName / 指定されたサーバーIDと接続しているCHOCOA上で、現在そのサービス名が使用されているかを取得します。
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOA上で、現在そのサービス名が使用されているかを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">サービス名</param>
        /// <returns>使用されている時は <c>true</c> 。</returns>
        public bool CheckScriptServiceName(int serverID, string scriptServiceName)
        {
            return this._chocoatCore.CheckScriptServiceName(serverID, scriptServiceName);
        }
        #endregion

        #region RegisterScript / 指定されたサーバーIDと接続しているCHOCOAに、スクリプトを登録します。
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOAに、スクリプトを登録します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="script">サービス名</param>
        /// <param name="isSave">保存指定</param>
        /// <param name="scriptServiceName">登録されたサービス名</param>
        /// <returns>登録された時は <c>true</c> 。</returns>
        public bool RegisterScript(int serverID, string script, bool isSave, out string scriptServiceName)
        {
            return this._chocoatCore.RegisterScript(serverID, script, isSave, out scriptServiceName);
        }
        #endregion

        #region DeleteScript
        /// <summary>
        /// 指定されたサービスのスクリプトを削除します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">サービス名</param>
        /// <returns>正常に削除されれば <c>true</c> 。</returns>
        public bool DeleteScript(int serverID, string scriptServiceName)
        {
            return this._chocoatCore.DeleteScript(serverID, scriptServiceName);
        }
        #endregion

        #region RegisterScriptFunction / 指定されたサーバーIDと接続しているCHOCOAに、スクリプトコマンドを登録します。このスクリプトコマンドは一時的なもので、CHOCOA終了時点で、登録が削除されます。
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOAに、スクリプトコマンドを登録します。このスクリプトコマンドは一時的なもので、CHOCOA終了時点で、登録が削除されます。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="script">サービス名</param>
        /// <param name="scriptServiceName">登録されたサービス名</param>
        /// <returns>登録された時は <c>true</c> 。</returns>
        public bool RegisterScriptFunction(int serverID, string script, out string scriptServiceName)
        {
            return this._chocoatCore.RegisterScriptFunction(serverID, script, out scriptServiceName);
        }
        #endregion

        #region ExecuteScript / 登録したスクリプトコマンドを実行します。
        /// <summary>
        /// 登録したスクリプトコマンドを実行します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">スクリプトサービス名</param>
        /// <param name="result">実行結果</param>
        /// <returns>正常に終了した場合は <c>true</c> 。</returns>
        public bool ExecuteScript(int serverID, string scriptServiceName, out string result)
        {
            return this._chocoatCore.ExecuteScript(serverID, scriptServiceName, out result);
        }
        #endregion

        #region SetVariable / 指定したサーバーの設定情報データを変更します。
        /// <summary>
        /// 指定したサーバーの設定情報データを変更します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="type">変更データタイプ</param>
        /// <param name="value">変更データ</param>
        /// <returns>正常に終了した場合は <c>true</c> 。</returns>
        public bool SetVariable(int serverID, string type, string value)
        {
            return this._chocoatCore.SetVariable(serverID, type, value);
        }
        #endregion

        #region GetVariable / 指定されたサーバーの設定情報のデータを取得します。
        /// <summary>
        /// 指定されたサーバーの設定情報のデータを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="type">取得するデータタイプ</param>
        /// <returns>取得した値。取得出来ない時は <c>null</c> 。</returns>
        public string GetVariable(int serverID, string type)
        {
            return this._chocoatCore.GetVariable(serverID, type);
        }
        #endregion

        //IRC情報周り
        #region GetServerList / 現在接続されているサーバーのリストを取得します。
        /// <summary>
        /// 現在接続されているサーバーのリストを取得します。
        /// </summary>
        /// <returns>接続されているサーバーのリスト</returns>
        public string[] GetConnectedServerList()
        {
            return this._chocoatCore.GetConnectedServerList();
        }
        #endregion

        #region GetChannelList / 指定されたサーバーの自分が入っているチャンネルのリストを取得します。
        /// <summary>
        /// 指定されたサーバーの自分が入っているチャンネルのリストを取得します。
        /// </summary>
        /// <param name="serverID">サーバーId</param>
        /// <returns>接続しているチャンネルのリスト</returns>
        public string[] GetChannelList(int serverID)
        {
            return this._chocoatCore.GetChannelList(serverID);
        }
        #endregion

        #region GetChannelUserList / 指定されたサーバーのユーザーリストを取得します。
        /// <summary>
        /// 指定されたサーバーのユーザーリストを取得します。
        /// </summary>
        /// <param name="serverID">接続に利用されるサーバーID</param>
        /// <param name="channel">取得するチャンネル</param>
        /// <returns>取得したユーザーのリスト</returns>
        public string[] GetChannelUserList(int serverID, string channel)
        {
            return this._chocoatCore.GetChannelUserList(serverID, channel);
        }
        #endregion

        //IRCコマンド周り
        #region SendPrivateMessage / プライベートメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// <summary>
        /// プライベートメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="channel">送信するチャンネル</param>
        /// <param name="message">送信するメッセージ</param>
        /// <returns>送信に成功した時は <c>true</c> 。</returns>
        public bool SendPrivateMessage(int serverID, string channel, string message)
        {
            return this._chocoatCore.SendPrivateMessage(serverID, channel, message);
        }
        #endregion

        #region SendNoticeMessage / Noticeメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// <summary>
        /// Noticeメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="channel">送信するチャンネル</param>
        /// <param name="message">送信するメッセージ</param>
        /// <returns>送信に成功した時は <c>true</c> 。</returns>
        public bool SendNoticeMessage(int serverID, string channel, string message)
        {
            return this._chocoatCore.SendNoticeMessage(serverID, channel, message);
        }
        #endregion

        #region ChangeNick / ニックネームを変更します。
        /// <summary>
        /// ニックネームを変更します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="newNick">新しいニックネーム</param>
        /// <returns>ニックネームが変更された時は <c>true</c> 。</returns>
        public bool ChangeNick(int serverID, string newNick)
        {
            return this._chocoatCore.ChangeNick(serverID, newNick);
        }
        #endregion

        //COMイベントをサポートするか
        #region SupportMessageEvent / メッセージイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// メッセージイベントをサポートするかを取得、または設定します。
        /// </summary>
        [Bindable(false)]
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("メッセージイベントをサポートするかを取得、または設定します。")]
        public bool SupportMessageEvent
        {
            get
            {
                return this._supportMessageEvent;
            }
            set
            {
                this._supportMessageEvent = value;
                this.SetEvents();
            }
        }
        #endregion

        #region SupportCTCPMessageEvent / CTCPイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// CTCPイベントをサポートするかを取得、または設定します。
        /// </summary>
        [Bindable(false)]
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("CTCPイベントをサポートするかを取得、または設定します。")]
        public bool SupportCTCPMessageEvent
        {
            get
            {
                return this._supportCTCPMessageEvent;
            }
            set
            {
                this._supportCTCPMessageEvent = value;
                this.SetEvents();
            }
        }
        #endregion

        #region SupportIRCEvent / IRCイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// IRCイベントをサポートするかを取得、または設定します。
        /// </summary>
        [Bindable(false)]
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("IRCイベントをサポートするかを取得、または設定します。")]
        public bool SupportIRCEvent
        {
            get
            {
                return this._supportIRCEvent;
            }
            set
            {
                this._supportIRCEvent = value;
                this.SetEvents();
            }
        }
        #endregion

        #region SupportScriptEvent / Scriptイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// Scriptイベントをサポートするかを取得、または設定します。
        /// </summary>
        [Bindable(false)]
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("Scriptイベントをサポートするかを取得、または設定します。")]
        public bool SupportScriptEvent
        {
            get
            {
                return this._supportScriptEvent;
            }
            set
            {
                this._supportScriptEvent = value;
                this.SetEvents();
            }
        }
        #endregion

        #region SupportNumericMessageEvent / Numericメッセージイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// Numericメッセージイベントをサポートするかを取得、または設定します。
        /// </summary>
        [Bindable(false)]
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("Numericメッセージイベントをサポートするかを取得、または設定します。")]
        public bool SupportNumericMessageEvent
        {
            get
            {
                return this._supportNumericMessageEvent;
            }
            set
            {
                this._supportNumericMessageEvent = value;
                this.SetEvents();
            }
        }
        #endregion

        #region SupportChocoaEvent / Chocoaイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// Chocoaイベントをサポートするかを取得、または設定します。
        /// </summary>
        [Bindable(false)]
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("CHOCOAイベントをサポートするかを取得、または設定します。")]
        public bool SupportChocoaEvent
        {
            get
            {
                return this._supportChocoaEvent;
            }
            set
            {
                this._supportChocoaEvent = value;
                this.SetEvents();
            }
        } 
        #endregion

        //初期化が終わるまで何もしないための仕掛け
        #region SetEvents / イベントのサポート状態を更新するとこ

        private bool _supportMessageEvent;
        private bool _supportCTCPMessageEvent;
        private bool _supportIRCEvent;
        private bool _supportScriptEvent;
        private bool _supportNumericMessageEvent;
        private bool _supportChocoaEvent;

        /// <summary>
        /// イベントのサポート状態を更新するとこ
        /// </summary>
        private void SetEvents()
        {
            if (!this._initializing && !this.DesignMode)
            {
                this._chocoatCore.SupportMessageEvent = this._supportMessageEvent;

                this._chocoatCore.SupportCTCPMessageEvent = this.SupportCTCPMessageEvent;

                this._chocoatCore.SupportIRCEvent = this.SupportIRCEvent;

                this._chocoatCore.SupportScriptEvent = this._supportScriptEvent;

                this._chocoatCore.SupportNumericMessageEvent = this._supportNumericMessageEvent;

                this._chocoatCore.SupportChocoaEvent = this._supportChocoaEvent;
            }
        }
        #endregion

        //イベント
        #region MessageEvent / CHOCOAがメッセージを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがメッセージを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがメッセージを受信した時に呼び出されます。")]
        public event MessageEventHandler MessageEvent;

        /// <summary>
        /// <see cref="MessageEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnMessageEvent(MessageEventArgs e)
        {
            if (this.MessageEvent != null)
            {
                this.MessageEvent(this, e);
            }
        }
        #endregion

        #region ScriptEvent / CHOCOAがスクリプトイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがスクリプトイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがスクリプトイベントを受信した時に呼び出されます。")]
        public event ScriptEventHandler ScriptEvent;

        /// <summary>
        /// <see cref="ScriptEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnScriptEvent(ScriptEventArgs e)
        {
            if (this.ScriptEvent != null)
            {
                this.ScriptEvent(this, e);
            }
        }
        #endregion

        #region IRCNickEvent / CHOCOAがNickイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがNickイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがNickイベントを受信した時に呼び出されます。")]
        public event IRCNickEventHandler IRCNickEvent;

        /// <summary>
        /// <see cref="IRCNickEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCNickEvent(IRCNickEventArgs e)
        {
            if (this.IRCNickEvent != null)
            {
                this.IRCNickEvent(this, e);
            }
        }
        #endregion

        #region IRCJoinEvent　/ CHOCOAがJoinイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがJoinイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがJoinイベントを受信した時に呼び出されます。")]
        public event IRCJoinEventHandler IRCJoinEvent;

        /// <summary>
        /// <see cref="IRCJoinEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCJoinEvent(IRCJoinEventArgs e)
        {
            if (this.IRCJoinEvent != null)
            {
                this.IRCJoinEvent(this, e);
            }
        }
        #endregion

        #region IRCTopicEvent / CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがTopicイベントを受信した時に呼び出されます。")]
        public event IRCTopicEventHandler IRCTopicEvent;

        /// <summary>
        /// <see cref="IRCTopicEvent"/> を発生させます。
        /// </summary>
        /// <param name="e"></param>
        private void OnIRCTopicEvent(IRCTopicEventArgs e)
        {
            if (this.IRCTopicEvent != null)
            {
                this.IRCTopicEvent(this, e);
            }
        }
        #endregion

        #region IRCNamesEvent / CHOCOAがNamesイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがNamesイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがNamesイベントを受信した時に呼び出されます。")]
        public event IRCNamesEventHandler IRCNamesEvent;

        /// <summary>
        /// <see cref="IRCNamesEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCNamesEvent(IRCNamesEventArgs e)
        {
            if (this.IRCNamesEvent != null)
            {
                this.IRCNamesEvent(this, e);
            }
        }
        #endregion

        #region IRCPartEvent / CHOCOAがPartイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがPartイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがPartイベントを受信した時に呼び出されます。")]
        public event IRCPartEventHandler IRCPartEvent;

        /// <summary>
        /// <see cref="IRCPartEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCPartEvent(IRCPartEventArgs e)
        {
            if (this.IRCPartEvent != null)
            {
                this.IRCPartEvent(this, e);
            }
        }
        #endregion

        #region IRCQuitEvent / CHOCOAがQuitイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがQuitイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがQuitイベントを受信した時に呼び出されます。")]
        public event IRCQuitEventHandler IRCQuitEvent;

        /// <summary>
        /// <see cref="IRCQuitEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCQuitEvent(IRCQuitEventArgs e)
        {
            if (this.IRCQuitEvent != null)
            {
                this.IRCQuitEvent(this, e);
            }
        }
        #endregion

        #region IRCModeEvent / CHOCOAがModeイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがModeイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがModeイベントを受信した時に呼び出されます。")]
        public event IRCModeEventHandler IRCModeEvent;

        /// <summary>
        /// <see cref="IRCModeEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCModeEvent(IRCModeEventArgs e)
        {
            if (this.IRCModeEvent != null)
            {
                this.IRCModeEvent(this, e);
            }
        }
        #endregion

        #region IRCKickEvent / CHOCOAがKickイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがKickイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがKickイベントを受信した時に呼び出されます。")]
        public event IRCKickEventHandler IRCKickEvent;

        /// <summary>
        /// <see cref="IRCKickEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCKickEvent(IRCKickEventArgs e)
        {
            if (this.IRCKickEvent != null)
            {
                this.IRCKickEvent(this, e);
            }
        }
        #endregion

        #region IRCInviteEvent / CHOCOAがInviteイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがInviteイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがInviteイベントを受信した時に呼び出されます。")]
        public event IRCInviteEventHandler IRCInviteEvent;

        /// <summary>
        /// <see cref="IRCInviteEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCInviteEvent(IRCInviteEventArgs e)
        {
            if (this.IRCInviteEvent != null)
            {
                this.IRCInviteEvent(this, e);
            }
        }
        #endregion

        #region IRCActionEvent / CHOCOAがActionイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがActionイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがActionイベントを受信した時に呼び出されます。")]
        public event IRCActionEventHandler IRCActionEvent;

        /// <summary>
        /// <see cref="IRCActionEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCActionEvent(IRCActionEventArgs e)
        {
            if (this.IRCActionEvent != null)
            {
                this.IRCActionEvent(this, e);
            }
        }
        #endregion

        #region IRCIsonEvent / CHOCOAがIsonイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがIsonイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがIsonイベントを受信した時に呼び出されます。")]
        public event IRCIsonEventHandler IRCIsonEvent;

        /// <summary>
        /// <see cref="IRCIsonEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCIsonEvent(IRCIsonEventArgs e)
        {
            if (this.IRCIsonEvent != null)
            {
                this.IRCIsonEvent(this, e);
            }
        }
        #endregion

        #region IRCWhoisEvent / CHOCOAがWhoisイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがWhoisイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがWhoisイベントを受信した時に呼び出されます。")]
        public event IRCWhoisEventHandler IRCWhoisEvent;

        private void OnIRCWhoisEvent(IRCWhoisEventArgs e)
        {
            if (this.IRCWhoisEvent != null)
            {
                this.IRCWhoisEvent(this, e);
            }
        }
        #endregion

        #region IRCWhoEvent / CHOCOAがWhoイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがWhoイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがWhoイベントを受信した時に呼び出されます。")]
        public event IRCWhoEventHandler IRCWhoEvent;

        private void OnIRCWhoEvent(IRCWhoEventArgs e)
        {
            if (this.IRCWhoEvent != null)
            {
                this.IRCWhoEvent(this, e);
            }
        }
        #endregion

        #region NumericMessageEvent / CHOCOAがニュメリックイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがニュメリックイベントを受信した時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAがニュメリックイベントを受信した時に呼び出されます。")]
        public event NumericMessageEventHandler NumericMessageEvent;

        /// <summary>
        /// <see cref="NumericMessageEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnNumericMessageEvent(NumericMessageEventArgs e)
        {
            if (this.NumericMessageEvent != null)
            {
                this.NumericMessageEvent(this, e);
            }
        }
        #endregion

        #region ChocoaWindowEvent / CHOCOAのウィンドウ状態が変更された時に呼び出されます。
        /// <summary>
        /// CHOCOAのウィンドウ状態が変更された時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("CHOCOAのウィンドウ状態が変更された時に呼び出されます。")]
        public event ChocoaWindowEventHandler ChocoaWindowEvent;

        /// <summary>
        /// <see cref="ChocoaWindowEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnChocoaWindowEvent(ChocoaWindowEventArgs e)
        {
            if (this.ChocoaWindowEvent != null)
            {
                this.ChocoaWindowEvent(this, e);
            }
        }
        #endregion

        //CCAPIではとれないが、CHOCOA Scriptを併用すれば、あるいわ
        #region GetNick / 現在利用しているNICKを取得します。
        /// <summary>
        /// 現在利用しているNICKを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <returns>取得したNICK。</returns>
        public string GetNick(int serverID)
        {
            return this._chocoatCore.GetNick(serverID);
        }
        #endregion
    }
}
