﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Reflection;
using System.Runtime.InteropServices;

namespace Chocoat
{
    /// <summary>
    /// CHOCOAのCCAPIをラップするクラス。基本的に外に出ない。
    /// </summary>
    /// <remarks>
    /// <para>
    /// CCAPIをラップするクラス。タイプライブラリからではなくて、わざわざ作っているのは、どうもタイプライブラリだと呼び出せないような…
    /// </para>
    /// <para>
    /// あと、相手がCOMなので、この中に完全に閉じ込めこともこのクラスのお仕事。
    /// </para>
    /// <note>CCAPIのすべてをサポートしないです。ほしいものはリファレンスを元に追加するか、言って。</note>
    /// </remarks>
    public sealed class Core : IDisposable, IChocoat, IChocoaEvents
    {
        /// <summary>
        /// COMへの参照
        /// </summary>
        private object _ccapi;

        /// <summary>
        /// イベントの接続Cookie
        /// </summary>
        private int _cookie;

        #region constructor / destructor
        /// <summary>
        /// Core の新しいインスタンスを生成します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1805:DoNotInitializeUnnecessarily")]
        public Core()
        {
            
        }

        /// <summary>
        /// クラスのファイナライザとして呼び出されます。
        /// </summary>
        ~Core()
        {
            Dispose(false);
        }
        #endregion

        #region Start / CCAPIの利用を開始します。
        /// <summary>
        /// CCAPIの利用を開始します。
        /// </summary>
        /// <returns>正常に開始出来たら、<c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2122:DoNotIndirectlyExposeMethodsWithLinkDemands")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public bool Start()
        {
            if (this._ccapi == null)
            {
                try
                {
                    //ちょこあさーん
                    Type chocoaComType = Type.GetTypeFromProgID("CHOCOA.Application");

                    if (chocoaComType != null)
                    {
                        this._ccapi = Activator.CreateInstance(chocoaComType);

                        if (this._ccapi != null)
                        {
                            //イベントの登録
                            System.Runtime.InteropServices.ComTypes.IConnectionPointContainer icpc = (System.Runtime.InteropServices.ComTypes.IConnectionPointContainer)this._ccapi;

                            Type typeiae = typeof(IChocoaEvents);

                            Attribute[] atributes = (Attribute[])typeiae.GetCustomAttributes(typeof(GuidAttribute), false);

                            string iid = ((GuidAttribute)atributes[0]).Value;

                            Guid guid = new Guid(iid);

                            //指定したインターフェースIDのインターフェースがあるか探す 
                            System.Runtime.InteropServices.ComTypes.IConnectionPoint icp;

                            icpc.FindConnectionPoint(ref guid, out icp);

                            if (icp != null)
                            {
                                icp.Advise(this, out this._cookie);
                            }

                            return true;
                        }
                    }
                }
                catch { }

                return false;
            }
            else
            {
                return true;
            }
        }
        #endregion

        #region Stop / CCAPIを停止します。
        /// <summary>
        /// CCAPIを停止します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2122:DoNotIndirectlyExposeMethodsWithLinkDemands")]
        public void Stop()
        {
            if (this._ccapi != null)
            {
                Marshal.ReleaseComObject(this._ccapi);
                this._ccapi = null;
            }
        } 
        #endregion

        #region IDisposable メンバ

        /// <summary>
        /// disposeが呼び出されたか
        /// </summary>
        private bool _disposed;

        /// <summary>
        /// Core のリソースを破棄します。
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// アンマネージリソースを含めたリソースを破棄します。
        /// </summary>
        /// <param name="disposing">マネージ リソースが破棄される場合 true、破棄されない場合は false です。</param>
        private void Dispose(bool disposing)
        {
            if (!this._disposed)
            {
                if (disposing)
                {
                    this.Stop();
                }
            }

            this._disposed = true;
        }
        #endregion

        #region InvokeMember / Invokeするとこの手抜き版
        /// <summary>
        /// Invokeするとこの手抜き版
        /// </summary>
        /// <param name="methodName">メソッド名</param>
        /// <param name="flags">バインディングフラグ</param>
        /// <param name="args">引数</param>
        /// <returns>メソッドの実行結果</returns>
        private object InvokeMember(string methodName, BindingFlags flags, params object[] args)
        {
            if (this._ccapi != null)
            {
                Type chocoaComType = this._ccapi.GetType();

                return chocoaComType.InvokeMember(
                    methodName,
                    flags,
                    null,
                    this._ccapi,
                    args,
                    CultureInfo.InvariantCulture);
            }

            return null;
        }

        /// <summary>
        /// Invokeするとこの手抜き版
        /// </summary>
        /// <param name="methodName">メソッド名</param>
        /// <param name="flags">バインディングフラグ</param>
        /// <param name="args">引数</param>
        /// <param name="mode">引数のモード</param>
        /// <returns>メソッドの実行結果</returns>
        private object InvokeMember(string methodName, BindingFlags flags, object[] args, ParameterModifier[] mode)
        {
            if (this._ccapi != null)
            {
                Type chocoaComType = this._ccapi.GetType();

                return chocoaComType.InvokeMember(
                    methodName,
                    flags,
                    null,
                    this._ccapi,
                    args,
                    mode,
                    CultureInfo.InvariantCulture,
                    null);
            }

            return null;
        } 
        #endregion

        #region CreateParameterModifier / パラメータの設定を作る手抜き
        /// <summary>
        /// パラメータの設定を作る手抜き
        /// </summary>
        /// <param name="modes">refになるものはtrueで。</param>
        /// <returns>生成されたパラメータ設定の配列</returns>
        private static ParameterModifier[] CreateParameterModifier(params bool[] modes)
        {
            ParameterModifier pm = new ParameterModifier(modes.Length);

            for (int i = 0; i < modes.Length; i++)
            {
                pm[i] = modes[i];
            }

            return new ParameterModifier[] { pm };
        } 
        #endregion

        //CHOCOAさん
        #region IsStart / CHOCOAクライアントのCCAPIが開始されているかを取得します。
        /// <summary>
        /// CHOCOAクライアントのCCAPIが開始されているかを取得します。
        /// </summary>
        public bool IsStart
        {
            get
            {
                return (this._ccapi != null);
            }
        }
        #endregion

        #region Version / インストールされているCHOCOAのバージョンを取得します。インストールされていない時は <c>cull</c> です。
        /// <summary>
        /// インストールされているCHOCOAのバージョンを取得します。インストールされていない時は <c>null</c> です。
        /// </summary>
        public string Version
        {
            get
            {
                if (this.IsStart)
                {
                    object[] args = { String.Empty };
                    ParameterModifier[] pm = CreateParameterModifier(true);

                    if ((bool)InvokeMember("GetVersion", BindingFlags.InvokeMethod, args, pm))
                    {
                        return (string)args[0];
                    }
                }

                return null;
            }
        }
        #endregion

        #region SettingServerCount / 設定されているサーバーの個数を取得します。
        /// <summary>
        /// 設定されているサーバーの個数を取得します。
        /// </summary>
        /// <returns>設定されているサーバーの個数</returns>
        public int SettingServerCount
        {
            get
            {
                if (this.IsStart)
                {
                    return (int)this.InvokeMember("GetSettingCount", BindingFlags.InvokeMethod, new object[0]);
                }

                return 0;
            }
        }
        #endregion

        #region HideWindow / ウィンドウを非表示にします。
        /// <summary>
        /// ウィンドウを非表示にします。
        /// </summary>
        public void HideWindow()
        {
            if (this.IsStart)
            {
                this.InvokeMember("HideWnd", BindingFlags.InvokeMethod, null);
            }
        }

        /// <summary>
        /// 指定されたサーバーIDと接続されているウィンドウを非表示にします。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        public void HideWindow(int serverID)
        {
            if (this.IsStart)
            {
                this.InvokeMember("HideWndEx", BindingFlags.InvokeMethod, serverID);
            }
        }
        #endregion

        #region ShowWindow / ウィンドウを表示します。
        /// <summary>
        /// ウィンドウを表示します。
        /// </summary>
        public void ShowWindow()
        {
            if (this.IsStart)
            {
                this.InvokeMember("ShowWnd", BindingFlags.InvokeMethod, null);
            }
        }
       
        /// <summary>
        /// 指定されたサーバーIDに接続されているウィンドウを表示します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        public void ShowWindow(int serverID)
        {
            if (this.IsStart)
            {
                this.InvokeMember("ShowWndEx", BindingFlags.InvokeMethod, serverID);
            }
        } 
        #endregion

        #region IsShowWindow / 指定されたサーバーと接続しているCHOCOAウィンドウが、表示状態にあるかをチェックします。
        /// <summary>
        /// 指定されたサーバーと接続しているCHOCOAウィンドウが、表示状態にあるかをチェックします。
        /// </summary>
        /// <param name="serverID">接続を確認するサーバーID。0を指定すると、すべてのウィンドウをチェックします。</param>
        /// <returns>表示され、接続されている時は <c>true</c> 。それ以外は <c>fakse</c> 。</returns>
        public bool IsShowWindow(int serverID)
        {
            if (this.IsStart)
            {
                return (bool)this.InvokeMember("IsShowWnd", BindingFlags.InvokeMethod, serverID);
            }

            return false;
        } 
        #endregion

        #region GetSettingServerName / 指定されたサーバーIDの設定名を取得します。
        /// <summary>
        /// 指定されたサーバーIDの設定名を取得します。
        /// </summary>
        /// <param name="serverID">取得するサーバーID</param>
        /// <returns>サーバー設定名。存在しない場合は <c>null</c> 。</returns>
        public string GetSettingServerName(int serverID)
        {
            if (this.IsStart)
            {
                string result = (string)this.InvokeMember("GetSettingAtEx", BindingFlags.InvokeMethod, serverID);

                if (String.IsNullOrEmpty(result))
                {
                    return null;
                }
                else
                {
                    return result;
                }
            }

            return null;
        } 
        #endregion

        #region GetSettingServerNames / 設定されているサーバー名を取得します。
        /// <summary>
        /// 設定されているサーバー名を取得します。
        /// </summary>
        /// <returns>取得したサーバー名の配列。</returns>
        public string[] GetSettingServerNames()
        {
            if (this.IsStart)
            {
                int count = this.SettingServerCount;

                List<string> result = new List<string>();

                for (int i = 0; i < count; i++)
                {
                    //1かららしい
                    string serverName = this.GetSettingServerName(i + 1);

                    if (!String.IsNullOrEmpty(serverName))
                    {
                        result.Add(serverName);
                    }
                }

                return result.ToArray();
            }

            return new string[0];
        } 
        #endregion

        #region FindServerID / CHOCOAから、サーバーの設定名を元に、CHOCOA内の管理IDを取得します。
        /// <summary>
        /// CHOCOAから、サーバーの設定名を元に、CHOCOA内の管理IDを取得します。
        /// </summary>
        /// <param name="settingName">CHOCOAのサーバー設定名。</param>
        /// <returns>取得した内部管理番号。正常に終了したら 0 以上のID。失敗時は 0 。</returns>
        public int FindServerID(string settingName)
        {
            if (this.IsStart)
            {
                return (int)this.InvokeMember("FindServEx", BindingFlags.InvokeMethod, settingName);
            }

            return 0;
        }
        #endregion

        #region IsConnect / 指定されたサーバーIDが接続状態にあるかを取得します。
        /// <summary>
        /// 指定されたサーバーIDが接続状態にあるかを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <returns>接続されていれば <c>true</c> 。接続されていない時は <c>false</c> 。</returns>
        public bool IsConnect(int serverID)
        {
            if (this.IsStart)
            {
                object obj = this.InvokeMember("IsConnect", BindingFlags.InvokeMethod, serverID);

                if (obj is Boolean)
                {
                    //リファレンスとちがうわけですよ
                    return (bool)obj;
                }
                else
                {
                    //一応、Intも
                    return ((int)obj != 0);
                }
            }

            return false;
        }
        #endregion

        #region CheckScriptServiceName / 指定されたサーバーIDと接続しているCHOCOA上で、現在そのサービス名が使用されているかを取得します。
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOA上で、現在そのサービス名が使用されているかを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">サービス名</param>
        /// <returns>使用されている時は <c>true</c> 。</returns>
        public bool CheckScriptServiceName(int serverID, string scriptServiceName)
        {
            if (this.IsStart)
            {
                object[] args = { serverID, scriptServiceName };
                ParameterModifier[] pm = CreateParameterModifier(false, true);

                return (bool)this.InvokeMember("ChkServiceName", BindingFlags.InvokeMethod, args, pm);
            }

            return false;
        } 
        #endregion

        #region RegisterScript / 指定されたサーバーIDと接続しているCHOCOAに、スクリプトを登録します。
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOAに、スクリプトを登録します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="script">サービス名</param>
        /// <param name="isSave">保存指定</param>
        /// <param name="scriptServiceName">登録されたサービス名</param>
        /// <returns>登録された時は <c>true</c> 。</returns>
        /// <remarks>
        /// <para>
        /// 詳しくは、CCAPIのヘルプをみればいいと思うよ！
        /// </para>
        /// </remarks>
        public bool RegisterScript(int serverID, string script, bool isSave, out string scriptServiceName)
        {
            scriptServiceName = String.Empty;

            if (this.IsStart)
            {
                object[] args = { serverID, script, scriptServiceName, isSave };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true, false);

                if ((bool)this.InvokeMember("RegScript", BindingFlags.InvokeMethod, args, pm))
                {
                    scriptServiceName = (string)args[2];
                    return true;
                }
            }

            return false;
        } 
        #endregion

        #region DeleteScript / 指定されたサービスのスクリプトを削除します。
        /// <summary>
        /// 指定されたサービスのスクリプトを削除します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">サービス名</param>
        /// <returns>正常に削除されれば <c>true</c> 。</returns>
        public bool DeleteScript(int serverID, string scriptServiceName)
        {
            if (this.IsStart)
            {
                object[] args = { serverID, scriptServiceName };
                ParameterModifier[] pm = CreateParameterModifier(false, true);

                return (bool)this.InvokeMember("DelScript", BindingFlags.InvokeMethod, args, pm);
            }

            return false;
        } 
        #endregion

        #region RegisterScriptFunction / 指定されたサーバーIDと接続しているCHOCOAに、スクリプトコマンドを登録します。このスクリプトコマンドは一時的なもので、CHOCOA終了時点で、登録が削除されます。
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOAに、スクリプトコマンドを登録します。このスクリプトコマンドは一時的なもので、CHOCOA終了時点で、登録が削除されます。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="script">サービス名</param>
        /// <param name="scriptServiceName">登録されたサービス名</param>
        /// <returns>登録された時は <c>true</c> 。</returns>
        public bool RegisterScriptFunction(int serverID, string script, out string scriptServiceName)
        {
            scriptServiceName = String.Empty;

            if (this.IsStart)
            {
                object[] args = { serverID, script, scriptServiceName };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true);

                if ((bool)this.InvokeMember("RegScriptFunc", BindingFlags.InvokeMethod, args, pm))
                {
                    scriptServiceName = (string)args[2];
                    return true;
                }
            }

            return false;
        }
        #endregion

        #region ExecuteScript / 登録したスクリプトコマンドを実行します。
        /// <summary>
        /// 登録したスクリプトコマンドを実行します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">スクリプトサービス名</param>
        /// <param name="result">実行結果</param>
        /// <returns>正常に終了した場合は <c>true</c> 。</returns>
        public bool ExecuteScript(int serverID, string scriptServiceName, out string result)
        {
            result = String.Empty;

            if (this.IsStart)
            {
                object[] args = { serverID, scriptServiceName, result };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true);

                if ((bool)this.InvokeMember("ExecScript", BindingFlags.InvokeMethod, args, pm))
                {
                    result = (string)args[2];
                    return true;
                }
            }

            return false;
        }
        #endregion

        #region SetVariable / 指定したサーバーの設定情報データを変更します。
        /// <summary>
        /// 指定したサーバーの設定情報データを変更します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="type">変更データタイプ</param>
        /// <param name="value">変更データ</param>
        /// <returns>正常に終了した場合は <c>true</c> 。</returns>
        public bool SetVariable(int serverID, string type, string value)
        {
            if (this.IsStart)
            {
                if ((bool)this.InvokeMember("SetVariableEx", BindingFlags.InvokeMethod, serverID, type, value))
                {
                    return true;
                }
            }

            return false;
        } 
        #endregion

        #region GetVariable / 指定されたサーバーの設定情報のデータを取得します。
        /// <summary>
        /// 指定されたサーバーの設定情報のデータを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="type">取得するデータタイプ</param>
        /// <returns>取得した値。取得出来ない時は <c>null</c> 。</returns>
        public string GetVariable(int serverID, string type)
        {
            if (this.IsStart)
            {
                object[] args = { serverID, type, String.Empty };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true);

                if ((bool)this.InvokeMember("GetVariable", BindingFlags.InvokeMethod, args, pm))
                {
                    return (string)args[2];
                }
            }

            return null;
        } 
        #endregion

        //IRC情報周り
        #region GetServerList / 現在接続されているサーバーのリストを取得します。
        /// <summary>
        /// 現在接続されているサーバーのリストを取得します。
        /// </summary>
        /// <returns>接続されているサーバーのリスト</returns>
        public string[] GetConnectedServerList()
        {
            if (this.IsStart)
            {
                object[] args = { String.Empty, 0 };
                ParameterModifier[] pm = CreateParameterModifier(true, true);

                if ((bool)this.InvokeMember("GetServList", BindingFlags.InvokeMethod, args, pm))
                {
                    //これ、嘘。実際に接続されているかは個別に判定しないとダメ
                    // > どうも、最初の設定値は返って来ちゃうみたい
                    List<string> result = new List<string>();

                    foreach (string serverName in ((string)args[0]).Split(' '))
                    {
                        int serverID = this.FindServerID(serverName);

                        if (this.IsConnect(serverID))
                        {
                            result.Add(serverName);
                        }
                    }

                    return result.ToArray();
                }
            }

            return new string[0];
        }
        #endregion

        #region GetChannelList / 指定されたサーバーの自分が入っているチャンネルのリストを取得します。
        /// <summary>
        /// 指定されたサーバーの自分が入っているチャンネルのリストを取得します。
        /// </summary>
        /// <param name="serverID">サーバーId</param>
        /// <returns>接続しているチャンネルのリスト</returns>
        public string[] GetChannelList(int serverID)
        {
            if (this.IsStart && this.IsConnect(serverID))
            {
                object[] args = { serverID, String.Empty, 0 };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true);

                if ((bool)this.InvokeMember("GetChanList", BindingFlags.InvokeMethod, args, pm))
                {
                    string[] list = ((string)args[1]).Split(' ');

                    return list;
                }
            }

            return new string[0];
        }
        #endregion

        #region GetChannelUserList / 指定されたサーバーのユーザーリストを取得します。
        /// <summary>
        /// 指定されたサーバーのユーザーリストを取得します。
        /// </summary>
        /// <param name="serverID">接続に利用されるサーバーID</param>
        /// <param name="channel">取得するチャンネル</param>
        /// <returns>取得したユーザーのリスト</returns>
        public string[] GetChannelUserList(int serverID, string channel)
        {
            if (this.IsStart && this.IsConnect(serverID))
            {
                object[] args = { serverID, channel, String.Empty, 0 };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true, true);

                if ((bool)this.InvokeMember("GetChanUserList", BindingFlags.InvokeMethod, args, pm))
                {
                    string[] list = ((string)args[2]).Split(' ');

                    return list;
                }
            }

            return new string[0];
        }
        #endregion

        //IRCコマンド周り
        #region SendPrivateMessage / プライベートメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// <summary>
        /// プライベートメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="channel">送信するチャンネル</param>
        /// <param name="message">送信するメッセージ</param>
        /// <returns>送信に成功した時は <c>true</c> 。</returns>
        public bool SendPrivateMessage(int serverID, string channel, string message)
        {
            if (this.IsStart && this.IsConnect(serverID))
            {
                object[] args = { serverID, channel, message };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true);

                return (bool)this.InvokeMember("PrivMessage", BindingFlags.InvokeMethod, args, pm);
            }

            return false;
        }
        #endregion

        #region SendNoticeMessage / Noticeメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// <summary>
        /// Noticeメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="channel">送信するチャンネル</param>
        /// <param name="message">送信するメッセージ</param>
        /// <returns>送信に成功した時は <c>true</c> 。</returns>
        public bool SendNoticeMessage(int serverID, string channel, string message)
        {
            if (this.IsStart && this.IsConnect(serverID))
            {
                object[] args = { serverID, channel, message };
                ParameterModifier[] pm = CreateParameterModifier(false, true, true);

                return (bool)this.InvokeMember("NoticeMessage", BindingFlags.InvokeMethod, args, pm);
            }

            return false;
        }
        #endregion

        #region ChangeNick / ニックネームを変更します。
        /// <summary>
        /// ニックネームを変更します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="newNick">新しいニックネーム</param>
        /// <returns>ニックネームが変更された時は <c>true</c> 。</returns>
        public bool ChangeNick(int serverID, string newNick)
        {
            if (this.IsStart && this.IsConnect(serverID))
            {
                object[] args = { serverID, newNick };
                ParameterModifier[] pm = CreateParameterModifier(false, true);

                return (bool)this.InvokeMember("Nick", BindingFlags.InvokeMethod, args, pm);
            }

            return false;
        }
        #endregion

        //COMイベントをサポートするか
        #region SupportMessageEvent / メッセージイベントをサポートするかを取得、または設定します。
        
        /// <summary>
        /// メッセージイベントをサポートするかを取得、または設定します。
        /// </summary>
        public bool SupportMessageEvent
        {
            set
            {
                this.InvokeMember("MessageEvent", BindingFlags.SetProperty, value);

                if (this.ChangeSupportMessageEvent != null)
                {
                    this.ChangeSupportMessageEvent(this, new SupportPropertyChangedEventArgs(value));
                }
            }
        } 

        /// <summary>
        /// 内部的な仕掛け
        /// </summary>
        internal event EventHandler<SupportPropertyChangedEventArgs> ChangeSupportMessageEvent;

        #endregion

        #region SupportCTCPMessageEvent / CTCPイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// CTCPイベントをサポートするかを取得、または設定します。
        /// </summary>
        public bool SupportCTCPMessageEvent
        {
            set
            {
                this.InvokeMember("CTCPMessageEvent", BindingFlags.SetProperty, value);

                if (this.ChangeSupportCTCPMessageEvent != null)
                {
                    this.ChangeSupportCTCPMessageEvent(this, new SupportPropertyChangedEventArgs(value));
                }
            }
        }

        /// <summary>
        /// 内部的な仕掛け
        /// </summary>
        internal event EventHandler<SupportPropertyChangedEventArgs> ChangeSupportCTCPMessageEvent;

        #endregion

        #region SupportIRCEvent / IRCイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// IRCイベントをサポートするかを取得、または設定します。
        /// </summary>
        public bool SupportIRCEvent
        {
            set
            {
                this.InvokeMember("IRCEvent", BindingFlags.SetProperty, value);

                if (this.ChangeSupportIRCEvent != null)
                {
                    this.ChangeSupportIRCEvent(this, new SupportPropertyChangedEventArgs(value));
                }
            }
        }

        /// <summary>
        /// 内部的な仕掛け
        /// </summary>
        internal event EventHandler<SupportPropertyChangedEventArgs> ChangeSupportIRCEvent;

        #endregion

        #region SupportScriptEvent / Scriptイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// Scriptイベントをサポートするかを取得、または設定します。
        /// </summary>
        public bool SupportScriptEvent
        {
            set
            {
                this.InvokeMember("ScriptEvent", BindingFlags.SetProperty, value);

                if (this.ChangeSupportScriptEvent != null)
                {
                    this.ChangeSupportScriptEvent(this, new SupportPropertyChangedEventArgs(value));
                }
            }
        }

        /// <summary>
        /// 内部的な仕掛け
        /// </summary>
        internal event EventHandler<SupportPropertyChangedEventArgs> ChangeSupportScriptEvent;

        #endregion

        #region SupportNumericMessageEvent / Numericメッセージイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// Numericメッセージイベントをサポートするかを取得、または設定します。
        /// </summary>
        public bool SupportNumericMessageEvent
        {
            set
            {
                this.InvokeMember("NumericMessageEvent", BindingFlags.SetProperty, value);

                if (this.ChangeSupportNumericMessageEvent != null)
                {
                    this.ChangeSupportNumericMessageEvent(this, new SupportPropertyChangedEventArgs(value));
                }
            }
        }

        /// <summary>
        /// 内部的な仕掛け
        /// </summary>
        internal event EventHandler<SupportPropertyChangedEventArgs> ChangeSupportNumericMessageEvent;

        #endregion

        #region SupportChocoaEvent / Chocoaイベントをサポートするかを取得、または設定します。
        /// <summary>
        /// Chocoaイベントをサポートするかを取得、または設定します。
        /// </summary>
        public bool SupportChocoaEvent
        {
            set
            {
                this.InvokeMember("ChocoaEvent", BindingFlags.SetProperty, value);

                if (this.ChangeSupportChocoaEvent != null)
                {
                    this.ChangeSupportChocoaEvent(this, new SupportPropertyChangedEventArgs(value));
                }
            }
        }

        /// <summary>
        /// 内部的な仕掛け
        /// </summary>
        internal event EventHandler<SupportPropertyChangedEventArgs> ChangeSupportChocoaEvent;

        #endregion

        #region IChocoaEvents メンバ
        /// <summary>
        /// メッセージイベントの受信
        /// </summary>
        /// <param name="SettingNM">サーバー設定名</param>
        /// <param name="Channel">チャンネル</param>
        /// <param name="Nick">ニックネーム</param>
        /// <param name="Address">マシンアドレス</param>
        /// <param name="MessageType">メッセージタイプ</param>
        /// <param name="Message">メッセージ</param>
        void IChocoaEvents.MessageEvent(string SettingNM, string Channel, string Nick, string Address, short MessageType, string Message)
        {
            Chocoat.MessageType messType = Chocoat.MessageType.PrivMessage;

            if (MessageType == 1)
            {
                messType = Chocoat.MessageType.NoticeMessage;
            }

            this.OnMessageEvent(new MessageEventArgs(SettingNM, Channel, Nick, Address, messType, Message));
        }

        /// <summary>
        /// CTCPイベントの受信
        /// </summary>
        /// <param name="SettingNM">サーバー設定名</param>
        /// <param name="Channel">チャンネル</param>
        /// <param name="Nick">ニックネーム</param>
        /// <param name="Address">マシンアドレス</param>
        /// <param name="MessageType">メッセージタイプ</param>
        /// <param name="Message">メッセージ</param>
        void IChocoaEvents.CTCPMessageEvent(string SettingNM, string Channel, string Nick, string Address, short MessageType, string Message)
        {
            Chocoat.MessageType messType = Chocoat.MessageType.CTCPMessage;

            if (MessageType == 1)
            {
                messType = Chocoat.MessageType.CTCPReplyMessage;
            }

            this.OnMessageEvent(new MessageEventArgs(SettingNM, Channel, Nick, Address, messType, Message));
        }

        /// <summary>
        /// このデリゲートは、ターゲット2.0なんであるけど、3.0なら用意されているのでいらん
        /// </summary>
        private delegate void Action<T1, T2>(T1 obj1, T2 obj2);

        /// <summary>
        /// イベントのマップのいれもの
        /// </summary>
        private Dictionary<int, Action<string, string>> _eventMap = null;

        /// <summary>
        /// IRCイベントの受信
        /// </summary>
        /// <param name="SettingNM">サーバー名</param>
        /// <param name="MessageType">メッセージタイプ</param>
        /// <param name="ReValue">通知内容</param>
        void IChocoaEvents.IRCEvent(string SettingNM, int MessageType, string ReValue)
        {
            this.CreateIRCEventMap();

            if (this._eventMap.ContainsKey(MessageType))
            {
                Action<string, string> action = this._eventMap[MessageType];

                action(SettingNM, ReValue);
            }
        }

        #region CreateIRCEventMap / IRCイベントのマップ
        /// <summary>
        /// IRCイベントのマップ
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1502:AvoidExcessiveComplexity")]
        private void CreateIRCEventMap()
        {
            if (this._eventMap == null)
            {
                this._eventMap = new Dictionary<int, Action<string, string>>();

                //イベントのマップ
                this._eventMap.Add(4, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCNickEvent(new IRCNickEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(5, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCJoinEvent(new IRCJoinEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(8, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCTopicEvent(new IRCTopicEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(9, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCNamesEvent(new IRCNamesEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(10, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCPartEvent(new IRCPartEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(11, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCQuitEvent(new IRCQuitEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(12, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCModeEvent(new IRCModeEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(14, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCKickEvent(new IRCKickEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(18, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCInviteEvent(new IRCInviteEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(22, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCActionEvent(new IRCActionEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(24, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCIsonEvent(new IRCIsonEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(25, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCWhoisEvent(new IRCWhoisEventArgs(settingServerName, returnValue));
                });

                this._eventMap.Add(27, delegate(string settingServerName, string returnValue)
                {
                    this.OnIRCWhoEvent(new IRCWhoEventArgs(settingServerName, returnValue));
                });
            }
        }
        #endregion

        /// <summary>
        /// スクリプトイベントの受信
        /// </summary>
        /// <param name="SettingNM">サーバー名</param>
        /// <param name="EventType">イベントタイプ</param>
        /// <param name="ServiceNM">サービス名</param>
        /// <param name="FuncNM">コマンド名</param>
        /// <param name="ReValue">実行結果</param>
        void IChocoaEvents.ScriptEvent(string SettingNM, short EventType, string ServiceNM, string FuncNM, string ReValue)
        {
            this.OnScriptEvent(new ScriptEventArgs(SettingNM, EventType, ServiceNM, FuncNM, ReValue));
        }

        /// <summary>
        /// ニュメリックメッセージの受信
        /// </summary>
        /// <param name="SettingNM">サーバー名</param>
        /// <param name="Numeric">ニュメリック値</param>
        /// <param name="Message">メッセージ</param>
        void IChocoaEvents.NumericMessageEvent(string SettingNM, int Numeric, string Message)
        {
            this.OnNumericMessageEvent(new NumericMessageEventArgs(SettingNM, Numeric, Message));
        }

        /// <summary>
        /// CHOCOAさんの状態が変更された時に呼び出される
        /// </summary>
        /// <param name="SettingNM">サーバー名。サーバー名は一意になるので、これでウィンドウを特定するしかない。</param>
        /// <param name="tp">イベントタイプ</param>
        /// <param name="vaData">データ</param>
        void IChocoaEvents.ChocoaEvent(string SettingNM, string tp, object vaData)
        {
            switch (tp.ToUpper(CultureInfo.InvariantCulture))
            {
                case "MOVE":
                case "SIZE":
                case "ACTIVEWINDOW":
                    int[] args = (int[])vaData;
                    this.OnChocoaWindowEvent(new ChocoaWindowEventArgs(SettingNM, args[0], args[1], args[2], args[3], args[4]));
                    break;
            }
        }
        #endregion

        //イベント
        #region MessageEvent / CHOCOAがメッセージを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがメッセージを受信した時に呼び出されます。
        /// </summary>
        public event MessageEventHandler MessageEvent;

        /// <summary>
        /// <see cref="MessageEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnMessageEvent(MessageEventArgs e)
        {
            if (this.MessageEvent != null)
            {
                this.MessageEvent(this, e);
            }
        } 
        #endregion

        #region ScriptEvent / CHOCOAがスクリプトイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがスクリプトイベントを受信した時に呼び出されます。
        /// </summary>
        public event ScriptEventHandler ScriptEvent;

        /// <summary>
        /// <see cref="ScriptEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnScriptEvent(ScriptEventArgs e)
        {
            if (this.ScriptEvent != null)
            {
                this.ScriptEvent(this, e);
            }
        } 
        #endregion

        #region IRCNickEvent / CHOCOAがNicイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがNicイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCNickEventHandler IRCNickEvent;

        /// <summary>
        /// <see cref="IRCNickEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCNickEvent(IRCNickEventArgs e)
        {
            if (this.IRCNickEvent != null)
            {
                this.IRCNickEvent(this, e);
            }
        }
        #endregion

        #region IRCJoinEvent　/ CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCJoinEventHandler IRCJoinEvent;

        /// <summary>
        /// <see cref="IRCJoinEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCJoinEvent(IRCJoinEventArgs e)
        {
            if (this.IRCJoinEvent != null)
            {
                this.IRCJoinEvent(this, e);
            }
        }
        #endregion

        #region IRCTopicEvent / CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCTopicEventHandler IRCTopicEvent;

        /// <summary>
        /// <see cref="IRCTopicEvent"/> を発生させます。
        /// </summary>
        /// <param name="e"></param>
        private void OnIRCTopicEvent(IRCTopicEventArgs e)
        {
            if (this.IRCTopicEvent != null)
            {
                this.IRCTopicEvent(this, e);
            }
        }
        #endregion

        #region IRCNamesEvent / CHOCOAがNamesイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがNamesイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCNamesEventHandler IRCNamesEvent;

        /// <summary>
        /// <see cref="IRCNamesEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCNamesEvent(IRCNamesEventArgs e)
        {
            if (this.IRCNamesEvent != null)
            {
                this.IRCNamesEvent(this, e);
            }
        }
        #endregion

        #region IRCPartEvent / CHOCOAがPartイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがPartイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCPartEventHandler IRCPartEvent;

        /// <summary>
        /// <see cref="IRCPartEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCPartEvent(IRCPartEventArgs e)
        {
            if (this.IRCPartEvent != null)
            {
                this.IRCPartEvent(this, e);
            }
        }
        #endregion

        #region IRCQuitEvent / CHOCOAがQuitイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがQuitイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCQuitEventHandler IRCQuitEvent;

        /// <summary>
        /// <see cref="IRCQuitEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCQuitEvent(IRCQuitEventArgs e)
        {
            if (this.IRCQuitEvent != null)
            {
                this.IRCQuitEvent(this, e);
            }
        }
        #endregion

        #region IRCModeEvent / CHOCOAがModeイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがModeイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCModeEventHandler IRCModeEvent;

        /// <summary>
        /// <see cref="IRCModeEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCModeEvent(IRCModeEventArgs e)
        {
            if (this.IRCModeEvent != null)
            {
                this.IRCModeEvent(this, e);
            }
        }
        #endregion

        #region IRCKickEvent / CHOCOAがKickイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがKickイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCKickEventHandler IRCKickEvent;

        /// <summary>
        /// <see cref="IRCKickEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCKickEvent(IRCKickEventArgs e)
        {
            if (this.IRCKickEvent != null)
            {
                this.IRCKickEvent(this, e);
            }
        }
        #endregion

        #region IRCInviteEvent / CHOCOAがInviteイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがInviteイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCInviteEventHandler IRCInviteEvent;

        /// <summary>
        /// <see cref="IRCInviteEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCInviteEvent(IRCInviteEventArgs e)
        {
            if (this.IRCInviteEvent != null)
            {
                this.IRCInviteEvent(this, e);
            }
        }
        #endregion

        #region IRCActionEvent / CHOCOAがActionイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがActionイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCActionEventHandler IRCActionEvent;

        /// <summary>
        /// <see cref="IRCActionEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCActionEvent(IRCActionEventArgs e)
        {
            if (this.IRCActionEvent != null)
            {
                this.IRCActionEvent(this, e);
            }
        }
        #endregion

        #region IRCIsonEvent / CHOCOAがIsonイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがIsonイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCIsonEventHandler IRCIsonEvent;

        /// <summary>
        /// <see cref="IRCIsonEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnIRCIsonEvent(IRCIsonEventArgs e)
        {
            if (this.IRCIsonEvent != null)
            {
                this.IRCIsonEvent(this, e);
            }
        }
        #endregion

        #region IRCWhoisEvent / CHOCOAがWhoisイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがWhoisイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCWhoisEventHandler IRCWhoisEvent;

        private void OnIRCWhoisEvent(IRCWhoisEventArgs e)
        {
            if (this.IRCWhoisEvent != null)
            {
                this.IRCWhoisEvent(this, e);
            }
        }
        #endregion

        #region IRCWhoEvent / CHOCOAがWhoイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがWhoイベントを受信した時に呼び出されます。
        /// </summary>
        public event IRCWhoEventHandler IRCWhoEvent;

        private void OnIRCWhoEvent(IRCWhoEventArgs e)
        {
            if (this.IRCWhoEvent != null)
            {
                this.IRCWhoEvent(this, e);
            }
        }
        #endregion

        #region NumericMessageEvent / CHOCOAがニュメリックイベントを受信した時に呼び出されます。
        /// <summary>
        /// CHOCOAがニュメリックイベントを受信した時に呼び出されます。
        /// </summary>
        public event NumericMessageEventHandler NumericMessageEvent;

        /// <summary>
        /// <see cref="NumericMessageEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnNumericMessageEvent(NumericMessageEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(e);
            if (this.NumericMessageEvent != null)
            {
                this.NumericMessageEvent(this, e);
            }
        } 
        #endregion

        #region ChocoaWindowEvent / CHOCOAのウィンドウ状態が変更された時に呼び出されます。
        /// <summary>
        /// CHOCOAのウィンドウ状態が変更された時に呼び出されます。
        /// </summary>
        public event ChocoaWindowEventHandler ChocoaWindowEvent;

        /// <summary>
        /// <see cref="ChocoaWindowEvent"/> を発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        private void OnChocoaWindowEvent(ChocoaWindowEventArgs e)
        {
            if (this.ChocoaWindowEvent != null)
            {
                this.ChocoaWindowEvent(this, e);
            }
        }
        #endregion

        //CCAPIではとれないが、CHOCOA Scriptを併用すれば、あるいわ
        #region GetNick / 現在利用しているNICKを取得します。
        /// <summary>
        /// 現在利用しているNICKを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <returns>取得したNICK。</returns>
        public string GetNick(int serverID)
        {
            string scriptName = "CHOCOAT_GET_NICK";
            string nickScript = String.Format(CultureInfo.InvariantCulture, @"/{0}
{{
    /comresult $me
}}", scriptName);

            string serviceName = null;
            string result = null;

            if (this.RegisterScriptFunction(serverID, nickScript, out serviceName))
            {
                if (!this.ExecuteScript(serverID, String.Format(CultureInfo.InvariantCulture, "/{0}.{1}", serviceName, scriptName), out result))
                {
                    result = null;
                }
            }

            return result;
        } 
        #endregion
    }
}
