﻿using System;

namespace Chocoat
{
    /// <summary>
    /// Choco@がサポートする機能の一覧。CCAPIのすべてではないです。
    /// </summary>
    public interface IChocoat
    {
        /// <summary>
        /// CCAPIを開始します。
        /// </summary>
        /// <returns>正常に開始出来たら <c>true</c> 。</returns>
        bool Start();

        /// <summary>
        /// CCAPIを停止します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1716:IdentifiersShouldNotMatchKeywords", MessageId = "Stop")]   //Stopwatchクラスとかに普通にあるくせに！
        void Stop();

        //CHOCOAさん
        /// <summary>
        /// CHOCOAクライアントのCCAPIが開始されているかを取得します。
        /// </summary>
        bool IsStart { get;}

        /// <summary>
        /// インストールされているCHOCOAのバージョンを取得します。インストールされていない時は <c>null</c> です。
        /// </summary>
        string Version { get; }

        /// <summary>
        /// 設定されているサーバーの個数を取得します。
        /// </summary>
        /// <returns>設定されているサーバーの個数</returns>
        int SettingServerCount { get; }

        /// <summary>
        /// ウィンドウを非表示にします。
        /// </summary>
        void HideWindow();
       
        /// <summary>
        /// 指定されたサーバーIDと接続されているウィンドウを非表示にします。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        void HideWindow(int serverID);
        
        /// <summary>
        /// ウィンドウを表示します。
        /// </summary>
        void ShowWindow();

        /// <summary>
        /// 指定されたサーバーIDに接続されているウィンドウを表示します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        void ShowWindow(int serverID);
        
        /// <summary>
        /// 指定されたサーバーと接続しているCHOCOAウィンドウが、表示状態にあるかをチェックします。
        /// </summary>
        /// <param name="serverID">接続を確認するサーバーID。0を指定すると、すべてのウィンドウをチェックします。</param>
        /// <returns>表示され、接続されている時は <c>true</c> 。それ以外は <c>fakse</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool IsShowWindow(int serverID);

        /// <summary>
        /// 指定されたサーバーIDの設定名を取得します。
        /// </summary>
        /// <param name="serverID">取得するサーバーID</param>
        /// <returns>サーバー設定名。存在しない場合は <c>null</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        string GetSettingServerName(int serverID);
        
        /// <summary>
        /// 設定されているサーバー名を取得します。
        /// </summary>
        /// <returns>取得したサーバー名の配列。</returns>
        string[] GetSettingServerNames();

        /// <summary>
        /// CHOCOAから、サーバーの設定名を元に、CHOCOA内の管理IDを取得します。
        /// </summary>
        /// <param name="settingName">CHOCOAのサーバー設定名。</param>
        /// <returns>取得した内部管理番号。正常に終了したら 0 以上のID。失敗時は 0 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        int FindServerID(string settingName);

        /// <summary>
        /// 指定されたサーバーIDが接続状態にあるかを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <returns>接続されていれば <c>true</c> 。接続されていない時は <c>false</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool IsConnect(int serverID);

        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOA上で、現在そのスクリプトサービス名が使用されているかを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">スクリプトサービス名</param>
        /// <returns>使用されている時は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool CheckScriptServiceName(int serverID, string scriptServiceName);
        
        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOAに、スクリプトを登録します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="script">サービス名</param>
        /// <param name="isSave">保存指定</param>
        /// <param name="scriptServiceName">登録されたサービス名</param>
        /// <returns>登録された時は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "3#")]
        bool RegisterScript(int serverID, string script, bool isSave, out string scriptServiceName);
        
        /// <summary>
        /// 指定されたサービスのスクリプトを削除します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">サービス名</param>
        /// <returns>正常に削除されれば <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool DeleteScript(int serverID, string scriptServiceName);

        /// <summary>
        /// 指定されたサーバーIDと接続しているCHOCOAに、スクリプトコマンドを登録します。このスクリプトコマンドは一時的なもので、CHOCOA終了時点で、登録が削除されます。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="script">サービス名</param>
        /// <param name="scriptServiceName">登録されたサービス名</param>
        /// <returns>登録された時は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "2#")]
        bool RegisterScriptFunction(int serverID, string script, out string scriptServiceName);
      
        /// <summary>
        /// 登録したスクリプトコマンドを実行します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="scriptServiceName">スクリプトサービス名</param>
        /// <param name="result">実行結果</param>
        /// <returns>正常に終了した場合は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "2#")]
        bool ExecuteScript(int serverID, string scriptServiceName, out string result);
       
        /// <summary>
        /// 指定したサーバーの設定情報データを変更します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="type">変更データタイプ</param>
        /// <param name="value">変更データ</param>
        /// <returns>正常に終了した場合は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool SetVariable(int serverID, string type, string value);
        
        /// <summary>
        /// 指定されたサーバーの設定情報のデータを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <param name="type">取得するデータタイプ</param>
        /// <returns>取得した値。取得出来ない時は <c>null</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        string GetVariable(int serverID, string type);
        

        //IRC情報周り
        /// <summary>
        /// 現在接続されているサーバーのリストを取得します。
        /// </summary>
        /// <returns>接続されているサーバーのリスト</returns>
        string[] GetConnectedServerList();

        /// <summary>
        /// 指定されたサーバーの自分が入っているチャンネルのリストを取得します。
        /// </summary>
        /// <param name="serverID">サーバーId</param>
        /// <returns>接続しているチャンネルのリスト</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        string[] GetChannelList(int serverID);
        
        /// <summary>
        /// 指定されたサーバーのユーザーリストを取得します。
        /// </summary>
        /// <param name="serverID">接続に利用されるサーバーID</param>
        /// <param name="channel">取得するチャンネル</param>
        /// <returns>取得したユーザーのリスト</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        string[] GetChannelUserList(int serverID, string channel);


        //IRCコマンド周り
        /// <summary>
        /// プライベートメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="channel">送信するチャンネル</param>
        /// <param name="message">送信するメッセージ</param>
        /// <returns>送信に成功した時は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool SendPrivateMessage(int serverID, string channel, string message);

        /// <summary>
        /// Noticeメッセージを指定したサーバーの指定したチャンネルに対して発言します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="channel">送信するチャンネル</param>
        /// <param name="message">送信するメッセージ</param>
        /// <returns>送信に成功した時は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool SendNoticeMessage(int serverID, string channel, string message);

        /// <summary>
        /// ニックネームを変更します。
        /// </summary>
        /// <param name="serverID">送信するサーバー</param>
        /// <param name="newNick">新しいニックネーム</param>
        /// <returns>ニックネームが変更された時は <c>true</c> 。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        bool ChangeNick(int serverID, string newNick);

        //COMイベントをサポートするか
        /// <summary>
        /// メッセージイベントをサポートするかを設定します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1044:PropertiesShouldNotBeWriteOnly")]
        bool SupportMessageEvent { set; }

        /// <summary>
        /// CTCPイベントをサポートするかを設定します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "CTCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1044:PropertiesShouldNotBeWriteOnly")]
        bool SupportCTCPMessageEvent { set; }

        /// <summary>
        /// IRCイベントをサポートするかを設定します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1044:PropertiesShouldNotBeWriteOnly")]
        bool SupportIRCEvent { set; }

        /// <summary>
        /// Scriptイベントをサポートするかを設定します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1044:PropertiesShouldNotBeWriteOnly")]
        bool SupportScriptEvent { set; }

        /// <summary>
        /// Numericメッセージイベントをサポートするかを設定します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1044:PropertiesShouldNotBeWriteOnly")]
        bool SupportNumericMessageEvent { set; }

        /// <summary>
        /// Chocoaイベントをサポートするかを取得、または設定します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1044:PropertiesShouldNotBeWriteOnly")]
        bool SupportChocoaEvent { set; }

        /// <summary>
        /// CHOCOAがメッセージを受信した時に呼び出されます。
        /// </summary>
        event MessageEventHandler MessageEvent;

        /// <summary>
        /// CHOCOAがスクリプトイベントを受信した時に呼び出されます。
        /// </summary>
        event ScriptEventHandler ScriptEvent;

        /// <summary>
        /// CHOCOAがNicイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCNickEventHandler IRCNickEvent;

        /// <summary>
        /// CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCJoinEventHandler IRCJoinEvent;

        /// <summary>
        /// CHOCOAがTopicイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCTopicEventHandler IRCTopicEvent;

        /// <summary>
        /// CHOCOAがNamesイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCNamesEventHandler IRCNamesEvent;

        /// <summary>
        /// CHOCOAがPartイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCPartEventHandler IRCPartEvent;

        /// <summary>
        /// CHOCOAがQuitイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCQuitEventHandler IRCQuitEvent;

        /// <summary>
        /// CHOCOAがModeイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCModeEventHandler IRCModeEvent;

        /// <summary>
        /// CHOCOAがKickイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCKickEventHandler IRCKickEvent;

        /// <summary>
        /// CHOCOAがInviteイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCInviteEventHandler IRCInviteEvent;

        /// <summary>
        /// CHOCOAがActionイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCActionEventHandler IRCActionEvent;

        /// <summary>
        /// CHOCOAがIsonイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCIsonEventHandler IRCIsonEvent;

        /// <summary>
        /// CHOCOAがWhoisイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCWhoisEventHandler IRCWhoisEvent;

        /// <summary>
        /// CHOCOAがWhoイベントを受信した時に呼び出されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
        event IRCWhoEventHandler IRCWhoEvent;

        /// <summary>
        /// CHOCOAがニュメリックイベントを受信した時に呼び出されます。
        /// </summary>
        event NumericMessageEventHandler NumericMessageEvent;

        /// <summary>
        /// CHOCOAのウィンドウ状態が変更された時に呼び出されます。
        /// </summary>
        event ChocoaWindowEventHandler ChocoaWindowEvent;

        /// <summary>
        /// 現在利用しているNICKを取得します。
        /// </summary>
        /// <param name="serverID">サーバーID</param>
        /// <returns>取得したNICK。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ID")]
        string GetNick(int serverID);
    }
}
