﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.NetworkInformation;
using System.Globalization;

namespace Chocoat.Utility
{      /// <summary>
    /// TcpTablesで利用される結果の戻り値のクラスです。
    /// </summary>
    /// <remarks>
    /// <para>
    /// IPv4しか対応しないんだけど、どうせ、CHOCOA は IPv6 の世界にいったら、動かなそうだしね。
    /// </para>
    /// </remarks>
    public class TcpRow
    {
        #region constructor
        /// <summary>
        /// Result の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="localAddr">ローカルアドレス</param>
        /// <param name="localPort">ローカルポート</param>
        /// <param name="remoteAddr">リモートアドレス</param>
        /// <param name="remotePort">リモートポート</param>
        /// <param name="state">状態</param>
        /// <remarks>
        /// <para>
        /// ライブラリ直で書く人以外は使えないコンストラクタだけど、まー、いないだろう。
        /// </para>
        /// </remarks>
        internal TcpRow(
            int localAddr,
            int localPort,
            int remoteAddr,
            int remotePort,
            int state)
        {
            this._localAddr = localAddr;
            this._localPort = localPort;
            this._remoteAddr = remoteAddr;
            this._remotePort = remotePort;
            this._state = state;
        }
        #endregion

        #region LocalAddress / ローカルアドレスを取得します。

        private int _localAddr;

        /// <summary>
        /// ローカルアドレスを取得します。
        /// </summary>
        public IPAddress LocalAddress
        {
            get
            {
                return new IPAddress(BitConverter.GetBytes(this._localAddr));
            }
        }
        #endregion

        #region LocalPort / ローカルポートを取得します。

        private int _localPort;

        /// <summary>
        /// ローカルポートを取得します。
        /// </summary>
        public int LocalPort
        {
            get
            {
                return GetPortNumber(this._localPort);
            }
        }
        #endregion

        #region RemoteAddress / リモートアドレスを取得します。

        private int _remoteAddr;

        /// <summary>
        /// リモートアドレスを取得します。
        /// </summary>
        public IPAddress RemoteAddress
        {
            get
            {
                return new IPAddress(BitConverter.GetBytes(this._remoteAddr));
            }
        }
        #endregion

        #region RemotePort / リモートポートを取得します。

        private int _remotePort;

        /// <summary>
        /// リモートポートを取得します。
        /// </summary>
        public int RemotePort
        {
            get
            {
                //RemoteAddrが0の場合は、Remoteポートも0にする
                if (this._remotePort == 0)
                {
                    return 0;
                }

                return GetPortNumber(this._remotePort);
            }
        }
        #endregion

        #region State / ポートのステータスを取得します。

        private int _state;

        /// <summary>
        /// ポートのステータスを取得します。
        /// </summary>
        public TcpState State
        {
            get
            {
                switch (this._state)
                {
                    case MIB_TCP_STATE_CLOSED:
                        return TcpState.Closed;
                    case MIB_TCP_STATE_LISTEN:
                        return TcpState.Listen;
                    case MIB_TCP_STATE_SYN_SENT:
                        return TcpState.SynSent;
                    case MIB_TCP_STATE_SYN_RCVD:
                        return TcpState.SynReceived;
                    case MIB_TCP_STATE_ESTAB:
                        return TcpState.Established;
                    case MIB_TCP_STATE_FIN_WAIT1:
                        return TcpState.FinWait1;
                    case MIB_TCP_STATE_FIN_WAIT2:
                        return TcpState.FinWait2;
                    case MIB_TCP_STATE_CLOSE_WAIT:
                        return TcpState.CloseWait;
                    case MIB_TCP_STATE_CLOSING:
                        return TcpState.Closing;
                    case MIB_TCP_STATE_LAST_ACK:
                        return TcpState.LastAck;
                    case MIB_TCP_STATE_TIME_WAIT:
                        return TcpState.TimeWait;
                    case MIB_TCP_STATE_DELETE_TCB:
                        return TcpState.DeleteTcb;
                    default:
                        return TcpState.Unknown;
                }
            }
        }
        #endregion

        #region ポートステータスはこんな感じなんだが、いる？

        /// <summary>
        /// TCP 接続が閉じています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "CLOSED")]
        public const short MIB_TCP_STATE_CLOSED = 0x1;

        /// <summary>
        /// TCP 接続のローカル エンドポイントがリモート エンドポイントからの接続要求を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "LISTEN")]
        public const short MIB_TCP_STATE_LISTEN = 0x2;

        /// <summary>
        /// TCP 接続のローカル エンドポイントが、リモート エンドポイントに同期 (SYN) 制御ビットを設定したセグメント ヘッダーを送信し、一致する接続要求を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "SYN")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "SENT")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        public const short MIB_TCP_STATE_SYN_SENT = 0x3;

        /// <summary>
        /// TCP 接続のローカル エンドポイントが接続要求の送受信を行い、受信確認を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "SYN")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "RCVD")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        public const short MIB_TCP_STATE_SYN_RCVD = 0x4;

        /// <summary>
        /// TCP ハンドシェイクが完了しています。接続が確立されているため、データを送信できます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ESTAB")]
        public const short MIB_TCP_STATE_ESTAB = 0x5;

        /// <summary>
        /// TCP 接続のローカル エンドポイントがリモート エンドポイントからの接続終了要求、または既に送信された接続終了要求の受信確認を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "WAIT")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "FIN")]
        public const short MIB_TCP_STATE_FIN_WAIT1 = 0x6;

        /// <summary>
        /// TCP 接続のローカル エンドポイントがリモート エンドポイントからの接続終了要求を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "WAIT")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "FIN")]
        public const short MIB_TCP_STATE_FIN_WAIT2 = 0x7;

        /// <summary>
        /// TCP 接続のローカル エンドポイントがローカル ユーザーからの接続終了要求を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "WAIT")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "CLOSE")]
        public const short MIB_TCP_STATE_CLOSE_WAIT = 0x8;

        /// <summary>
        /// TCP 接続のローカル エンドポイントが既に送信された接続終了要求の受信確認を待機しています。 
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "CLOSING")]
        public const short MIB_TCP_STATE_CLOSING = 0x9;

        /// <summary>
        /// TCP 接続のローカル エンドポイントが既に送信された接続終了要求の最終受信確認を待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "LAST")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "ACK")]
        public const short MIB_TCP_STATE_LAST_ACK = 0xA;

        /// <summary>
        /// TCP 接続のローカル エンドポイントが、リモート エンドポイントが確実に接続終了要求の受信確認を受信できるだけの期間、待機しています。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "WAIT")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TIME")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        public const short MIB_TCP_STATE_TIME_WAIT = 0xB;

        /// <summary>
        /// TCP 接続の伝送制御バッファ (TCB: Transmission Control Buffer) が削除されます。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1707:IdentifiersShouldNotContainUnderscores")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCP")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "TCB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STATE")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "MIB")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "DELETE")]
        public const short MIB_TCP_STATE_DELETE_TCB = 0xC;

        #endregion

        /// <summary>
        /// ポート変換のためのヘルパ
        /// </summary>
        /// <param name="port">ポート</param>
        /// <returns>取得したポート</returns>
        private static int GetPortNumber(int port)
        {
            return port / 256 + (port % 256) * 256;
        }

        /// <summary>
        /// オブジェクトを文字列で表します。
        /// </summary>
        /// <returns>文字列</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InvariantCulture,
                "{0}:{1} - {2}:{3} {4}",
                this.LocalAddress, this.LocalPort, this.RemoteAddress, this.RemotePort, this.State);
        }
    }
}
