﻿using System;
using System.Globalization;
using System.Runtime.InteropServices;

namespace Chocoat.Utility
{
    /// <summary>
    /// 利用中のポートの情報を保持するTcpRowのコレクションクラスです。
    /// </summary>
    /// <remarks>
    /// <para>
    /// System.Net.NetworkInformation にある IPGlobalProperties だとActive分しかでねーの？
    /// めんどくちぃなぁ。なんかあるのかなぁ。
    /// </para>
    /// </remarks>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1058:TypesShouldNotExtendCertainBaseTypes")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1039:ListsAreStronglyTyped")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1035:ICollectionImplementationsHaveStronglyTypedMembers")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1010:CollectionsShouldImplementGenericInterface")]
    public sealed class TcpTable : System.Collections.CollectionBase
    {
        #region private constructor
        /// <summary>
        /// TcpTable はインスタンス化出来ません。
        /// </summary>
        /// <remarks>
        /// <see cref="Current"/> で、取得してください。
        /// </remarks>
        private TcpTable()
        {
        }
        #endregion

        #region NativeMethods / よい子は、こうやってくくるんだ
        /// <summary>
        /// よい子は、こうやってくくるんだ
        /// </summary>
        internal static class NativeMethods
        {
            [DllImport("iphlpapi.dll", CharSet = CharSet.Auto)]
            public extern static int GetTcpTable(IntPtr pTcpTable, ref int pdwSize, [MarshalAs(UnmanagedType.Bool)] bool bOrder);

            [StructLayout(LayoutKind.Sequential)]
            public struct MIB_TCPROW
            {
                public int dwState;
                public int dwLocalAddr;
                public int dwLocalPort;
                public int dwRemoteAddr;
                public int dwRemotePort;
            }

            //ここの戻り値、不安…
            public const short NO_ERROR = 0x0;
            public const short ERROR_NOT_SUPPORTED = 0x32;
            public const short ERROR_INVALID_PARAMETER = 0x57;
            public const short ERROR_INSUFFICIENT_BUFFER = 0x70;
        }
        #endregion

        #region GetTcpTable / Tcp接続の状態を取得します。
        /// <summary>
        /// Tcp接続の状態を取得します。
        /// </summary>
        /// <returns>取得した状態</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2122:DoNotIndirectlyExposeMethodsWithLinkDemands")]
        public static TcpTable Current
        {
            get
            {
                int pdwSize = 0;

                IntPtr pTcpTable = IntPtr.Zero;

                TcpTable result = new TcpTable();

                try
                {
                    if (NativeMethods.GetTcpTable(pTcpTable, ref pdwSize, false) != NativeMethods.ERROR_INSUFFICIENT_BUFFER)
                    {
                        //手抜き
                        pTcpTable = Marshal.AllocHGlobal(pdwSize);

                        //TCPの一覧を取得
                        if (NativeMethods.GetTcpTable(pTcpTable, ref pdwSize, true) == NativeMethods.NO_ERROR)
                        {
                            //手抜き
                            int rowCount = Marshal.ReadInt32(pTcpTable);

                            for (int i = 0; i < rowCount; i++)
                            {
                                //1列ずつ取得
                                IntPtr pTcpRow = new IntPtr(pTcpTable.ToInt32() + 4 + (i * Marshal.SizeOf(typeof(NativeMethods.MIB_TCPROW))));

                                NativeMethods.MIB_TCPROW tcpRow = new NativeMethods.MIB_TCPROW();
                                tcpRow = (NativeMethods.MIB_TCPROW)Marshal.PtrToStructure(pTcpRow, typeof(NativeMethods.MIB_TCPROW));

                                //取得結果
                                result.List.Add(new TcpRow(
                                    tcpRow.dwLocalAddr,
                                    tcpRow.dwLocalPort,
                                    tcpRow.dwRemoteAddr,
                                    tcpRow.dwRemotePort,
                                    tcpRow.dwState));
                            }
                        }
                    }
                }
                finally
                {
                    Marshal.FreeHGlobal(pTcpTable);
                }

                return result;
            }
        }
        #endregion

        #region indexer
        /// <summary>
        /// 指定したインデックスにある要素を取得します。
        /// C# では、このプロパティは TcpTable クラスのインデクサになります。
        /// </summary>
        public TcpRow this[int index]
        {
            get
            {
                return (TcpRow)List[index];
            }
        }
        #endregion

        #region override
        /// <summary>
        /// 挿入時に呼び出されるもの。一致しないと例外なげます。
        /// </summary>
        /// <param name="index">位置</param>
        /// <param name="value">DataAccessRole</param>
        protected override void OnInsert(int index, Object value)
        {
            if (value.GetType() != typeof(TcpRow))
            {
                ThrowArgumentException("value", value);
            }
        }

        /// <summary>
        /// 削除時に呼び出されるもの。
        /// </summary>
        /// <param name="index">位置</param>
        /// <param name="value">DataAccessRole</param>
        protected override void OnRemove(int index, Object value)
        {
            if (value.GetType() != typeof(TcpRow))
            {
                ThrowArgumentException("value", value);
            }
        }

        /// <summary>
        /// Set時に呼び出されるもの。
        /// </summary>
        /// <param name="index">位置</param>
        /// <param name="oldValue">古いの</param>
        /// <param name="newValue">新しいの</param>
        protected override void OnSet(int index, Object oldValue, Object newValue)
        {
            if (newValue.GetType() != typeof(TcpRow))
            {
                ThrowArgumentException("newValue", newValue);
            }
        }

        /// <summary>
        /// OnValidate
        /// </summary>
        /// <param name="value">DataAccessRole</param>
        protected override void OnValidate(Object value)
        {
            if (value.GetType() != typeof(TcpRow))
            {
                ThrowArgumentException("value", value);
            }
        }

        /// <summary>
        /// 例外をThrowしてくれるだけのメソッド
        /// </summary>
        /// <param name="argName">引数の名前</param>
        /// <param name="value">実際の値</param>
        private static void ThrowArgumentException(string argName, object value)
        {
            throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, Properties.Resources.CollectionOnMethodArgumentExceptionMessage, argName, typeof(TcpRow), value.GetType()), argName);
        }
        #endregion

        //ここから下はChocoat用のカスタム
        //ここ以前は、別に普通に他のところでも使えるよ
        #region AlreadyPortUse / 指定されたポートがすでに利用されているかを取得します。
        /// <summary>
        /// 指定されたポートがすでに利用されているかを取得します。
        /// </summary>
        /// <param name="port">チェックするポート</param>
        /// <returns>利用されていれば <c>true</c> 。</returns>
        public bool AlreadyPortUse(int port)
        {
            foreach (TcpRow row in this.List)
            {
                if (row.LocalPort == port)
                {
                    return true;
                }
            }

            return false;
        }
        #endregion
    }
}