﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// エフェクト専用のコントロール
    /// </summary>
    [DefaultEvent("CheckedChanged")]
    [DefaultProperty("ErosionRateControl")]
    public partial class EffectControl : UserControl
    {
        #region constructor
        /// <summary>
        /// EffectControl の新しいインスタンスを生成します。
        /// </summary>
        public EffectControl()
        {
            InitializeComponent();

            //自動追加されると困る
            this.dgvEffect.AutoGenerateColumns = false;

            //コンボ使うためのチェック
            if (!this.dsDataSet.CharacterEffects.Columns.Contains("chk"))
            {
                this.dsDataSet.CharacterEffects.Columns.Add("chk", typeof(bool));
            }

            //コンボの生成
            foreach (DataGridViewComboBoxColumn typeCombo in new DataGridViewComboBoxColumn[]{
                this.diceTypeDataGridViewComboBoxColumn,
                this.bonusTypeDataGridViewComboBoxColumn,
                this.critTypeDataGridViewComboBoxColumn,
            })
            {
                typeCombo.DataSource = EnumStringAttribute.GetEnumValueLabelArray(typeof(EffectTypes));
                typeCombo.ValueMember = "Key";
                typeCombo.DisplayMember = "Value";
            }

            foreach (DataGridViewComboBoxColumn signCombo in new DataGridViewComboBoxColumn[] {
                this.diceSignDataGridViewComboBoxColumn,
                this.bonusSignDataGridViewComboBoxColumn,
                this.critSignDataGridViewComboBoxColumn,
            })
            {
                signCombo.DataSource = EnumStringAttribute.GetEnumValueLabelArray(typeof(EffectSign));
                signCombo.ValueMember = "Key";
                signCombo.DisplayMember = "Value";
            }
        } 
        #endregion

        #region BindDataTable / グリッドに対して、ソースをバインドします。
        /// <summary>
        /// グリッドに対して、ソースをバインドします。
        /// </summary>
        /// <param name="source">バインドするデータテーブル</param>
        public void BindDataTable(CharacterDataSet.CharacterEffectsDataTable source)
        {
            //念のため、コピーをマージする
            this.dsDataSet.CharacterEffects.Clear();
            this.dsDataSet.CharacterEffects.Merge(source.Copy());

            //コンボ使うためのチェック
            if (!this.dsDataSet.CharacterEffects.Columns.Contains("chk"))
            {
                this.dsDataSet.CharacterEffects.Columns.Add("chk", typeof(bool));
            }

            this.CalcDiceRoll();
        } 
        #endregion

        #region UpdateDataTable / 指定されたソースの中に、データをupdateします。
        /// <summary>
        /// 指定されたソースの中に、データをupdateします。
        /// </summary>
        /// <param name="source"></param>
        public void UpdateDataTable(CharacterDataSet.CharacterEffectsDataTable source)
        {
            source.Merge(this.dsDataSet.CharacterEffects);
        } 
        #endregion

        #region ErosionRateControl / 侵蝕率変更コントロールを取得、または設定します。

        ErosionRateControl _erosionRateControl;

        /// <summary>
        /// 侵蝕率変更コントロールを取得、または設定します。
        /// </summary>
        [Browsable(true)]
        public ErosionRateControl ErosionRateControl
        {
            get
            {
                return this._erosionRateControl;
            }
            set
            {
                this._erosionRateControl = value;
                this._erosionRateControl.ErosionRateChanged += delegate(object sender, ErosionRateChangedEventArgs e)
                {
                    this.CalcDiceRoll();
                };
            }
        }
        #endregion

        #region dgvEffect_CellContentClick / チェックした
        /// <summary>
        /// チェックした
        /// </summary>
        private void dgvEffect_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex > -1 && e.ColumnIndex == 0)
            {
                CharacterDataSet.CharacterEffectsRow row = GetBindingRow(this.dgvEffect.Rows[e.RowIndex]);

                bool nowState = false;

                if (Convert.IsDBNull(row["chk"]))
                {
                    nowState = false;
                }
                else
                {
                    nowState = (bool)row["chk"];
                }

                row["chk"] = !nowState;

                this.CalcDiceRoll();
            }
        }
        #endregion

        #region btnAddErosionRate_Click / 侵蝕値の加算
        /// <summary>
        /// 侵蝕値の加算
        /// </summary>
        private void btnAddErosionRate_Click(object sender, EventArgs e)
        {
            if (this.txtAddErosionRate.TextLength > 0)
            {
                this.ErosionRateControl.AddErosionRate(this.TotalErossionRate);
            }
        }
        #endregion

        #region chkEffectViewShort_CheckedChanged / 表示をショートする
        /// <summary>
        /// 表示をショートする
        /// </summary>
        private void chkEffectViewShort_CheckedChanged(object sender, EventArgs e)
        {
            bool visible = !this.chkEffectViewShort.Checked;

            this.timingDataGridViewTextBoxColumn.Visible = visible;
            this.skillDataGridViewTextBoxColumn.Visible = visible;
            this.challengeLevelDataGridViewTextBoxColumn.Visible = visible;
            this.targetDataGridViewTextBoxColumn.Visible = visible;
            this.rangeDataGridViewTextBoxColumn.Visible = visible;
        }
        #endregion

        #region btnEffectAdd_Click /エフェクトの追加
        /// <summary>
        /// エフェクトの追加
        /// </summary>
        private void btnEffectAdd_Click(object sender, EventArgs e)
        {
            DataRowView rowView = (DataRowView)this.bndEffect.AddNew();
            CharacterDataSet.CharacterEffectsRow row = (CharacterDataSet.CharacterEffectsRow)rowView.Row;

            using (AddEffect frm = new AddEffect(row))
            {
                if (frm.ShowDialog() == DialogResult.OK)
                {
                    //PKIDと連番
                    if (dsDataSet.CharacterDataMain.Count == 0)
                    {
                        //これPKIDないから、適当に
                        row.PKID = "0";
                    }
                    else
                    {
                        row.PKID = this.dsDataSet.CharacterDataMain[0].PKID;
                    }

                    object obj = this.dsDataSet.CharacterEffects.Compute("MAX(SerialNo)", null);
                    int maxSerialNo = (obj == null || Convert.IsDBNull(obj)) ? 0 : Convert.ToInt32(obj);
                    row.SerialNo = maxSerialNo + 1;

                    this.bndEffect.EndEdit();
                }
                else
                {
                    this.bndEffect.CancelEdit();
                }
                this.bndEffect.ResetBindings(false);
            }
        }
        #endregion

        #region btnEffectModify_Click / エフェクトの修正
        /// <summary>
        /// エフェクトの修正
        /// </summary>
        private void btnEffectModify_Click(object sender, EventArgs e)
        {
            if (this.dgvEffect.SelectedRows.Count > 0)
            {
                DataGridViewRow gvr = this.dgvEffect.SelectedRows[0];

                CharacterDataSet.CharacterEffectsRow row = this.GetBindingRow(gvr);

                using (AddEffect frm = new AddEffect(row))
                {
                    if (frm.ShowDialog() == DialogResult.OK)
                    {
                        this.bndEffect.EndEdit();
                        this.bndEffect.ResetBindings(false);
                    }
                }
            }
        }
        #endregion

        #region btnEffectDelete_Click / エフェクトの削除
        /// <summary>
        /// エフェクトの削除
        /// </summary>
        private void btnEffectDelete_Click(object sender, EventArgs e)
        {
            if (this.dgvEffect.SelectedRows.Count > 0)
            {
                DataGridViewRow gvr = this.dgvEffect.SelectedRows[0];

                CharacterDataSet.CharacterEffectsRow row = this.GetBindingRow(gvr);

                if (MessageBox.Show(
                    String.Format("{0} を削除します。", row.EffectName),
                    "確認",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    row.Delete();
                    this.bndEffect.ResetBindings(false);
                }
            }
        }
        #endregion

        //エフェクト部分のダイスの生成
        #region CreateRowDice / 指定されたグリッドの行にバインドされているデータから、ダイスのデータを生成します。
        /// <summary>
        /// 指定されたグリッドの行にバインドされているデータから、ダイスのデータを生成します。
        /// </summary>
        /// <param name="gridRow">グリッドの行</param>
        /// <returns>取得したダイスデータ</returns>
        private DiceRoll CreateRowDice(DataGridViewRow gridRow)
        {
            CharacterDataSet.CharacterEffectsRow row = GetBindingRow(gridRow);
            return this.CreateRowDice(row);
        }

        /// <summary>
        /// 指定されたグリッドの行にバインドされているデータから、ダイスのデータを生成します。
        /// </summary>
        /// <param name="gridRow">グリッドの行</param>
        /// <returns>取得したダイスデータ</returns>
        private DiceRoll CreateRowDice(CharacterDataSet.CharacterEffectsRow row)
        {
            DiceRoll result = new DiceRoll();

            result.Dice = row.DiceBase + (ConvertEffectType(row.EffectLevel, row.DiceType) * row.DiceScale + row.DiceAdd) * row.DiceSign;
            result.Bonus = row.BonusBase + (ConvertEffectType(row.EffectLevel, row.BonusType) * row.BonusScale + row.BonusAdd) * row.BonusSign;
            result.Critical = 10 - row.CritBase + (ConvertEffectType(row.EffectLevel, row.CritType) * row.CritScale + row.CritAdd) * row.CritSign;

            if (row.CritUnderLimit > result.Critical)
            {
                result.Critical = row.CritUnderLimit;
            }

            return result;
        }

        #region ConvertEffectType / エフェクトスケールの変換
        /// <summary>
        /// エフェクトスケールの変換
        /// </summary>
        /// <param name="effectLevel">エフェクトレベル</param>
        /// <param name="effectScale">エフェクトスケール</param>
        /// <returns>変換後の数値</returns>
        private int ConvertEffectType(int effectLevel, int effectScale)
        {
            EffectTypes effectType = (EffectTypes)Enum.Parse(typeof(EffectTypes), Enum.GetName(typeof(EffectTypes), effectScale));

            switch (effectType)
            {
                case EffectTypes.Zero:
                    return 0;
                case EffectTypes.Plus:
                    return 1;
                case EffectTypes.Minus:
                    return -1;
                case EffectTypes.PlusLevel:
                    return effectLevel + this.ErosionRateControl.EffectLevelBonus;
                case EffectTypes.MinusLevel:
                    return (effectLevel + this.ErosionRateControl.EffectLevelBonus) * -1;
                default:
                    throw new ArgumentOutOfRangeException("effectScale");
            }
        }
        #endregion

        #region GetBindingRow / 指定されたグリッドの行にバインドされている行のデータを取得します。
        /// <summary>
        /// 指定されたグリッドの行にバインドされている行のデータを取得します。
        /// </summary>
        /// <param name="gridRow">グリッドの行</param>
        /// <returns>バインドされている行</returns>
        private CharacterDataSet.CharacterEffectsRow GetBindingRow(DataGridViewRow gridRow)
        {
            DataRowView rowView = (DataRowView)gridRow.DataBoundItem;
            return (CharacterDataSet.CharacterEffectsRow)rowView.Row;
        }
        #endregion

        #endregion

        //エフェクト分の設定のプロパティ
        #region OnCheckedChanged CheckedChanged / チェック状態が変更された時に発生します。
        [Browsable(true)]
        [Description("チェック状態が変更された時に発生します。")]
        public event EventHandler CheckedChanged;

        /// <summary>
        /// <see cref="CheckedChanged"/> イベントを発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        protected void OnCheckedChanged(EventArgs e)
        {
            if (this.CheckedChanged != null)
            {
                this.CheckedChanged(this, e);
            }
        } 
        #endregion

        #region SelectedEffectNames / 選択されているエフェクト名を取得します。

        List<string> _selectedEffectList = new List<string>();

        /// <summary>
        /// 選択されているエフェクト名を取得します。
        /// </summary>
        [Browsable(false)]
        public string[] SelectedEffectNames
        {
            get
            {
                return this._selectedEffectList.ToArray();
            }
        } 
        #endregion

        #region Dice / 選択されたエフェクトのトータルのダイス数を取得します。
        /// <summary>
        /// 選択されたエフェクトのトータルのダイス数を取得します。
        /// </summary>
        [Browsable(false)]
        public int Dice
        {
            get;
            private set;
        } 
        #endregion

        #region Bonus / 選択されたエフェクトのトータルのボーナス数を取得します。
        /// <summary>
        /// 選択されたエフェクトのトータルのボーナス数を取得します。
        /// </summary>
        [Browsable(false)]
        public int Bonus
        {
            get;
            private set;
        } 
        #endregion

        #region MinusCritical / 選択されたエフェクトのトータルのクリティカル減少値を取得します。
        /// <summary>
        /// 選択されたエフェクトのトータルのクリティカル減少値を取得します。
        /// </summary>
        [Browsable(false)]
        public int MinusCritical
        {
            get;
            private set;
        } 
        #endregion

        #region UnderCritical / 選択されたエフェクトの下限値を取得します。
        /// <summary>
        /// 選択されたエフェクトの下限値を取得します。
        /// </summary>
        [Browsable(false)]
        public int UnderCritical
        {
            get;
            private set;
        } 
        #endregion

        #region TotalErossionRate / 選択されたエフェクトの侵蝕値のトータルを取得します。
        /// <summary>
        /// 選択されたエフェクトの侵蝕値のトータルを取得します。
        /// </summary>
        [Browsable(false)]
        public int TotalErossionRate
        {
            get;
            private set;
        } 
        #endregion

        #region CalcDiceRoll / ダイスの数を数えるところ
        /// <summary>
        /// ダイスの数を数えるところ
        /// </summary>
        public void CalcDiceRoll()
        {
            //エフェクト分
            this.Dice = 0;
            this.Bonus = 0;
            this.UnderCritical = 10;
            this.MinusCritical = 0;

            this.TotalErossionRate = 0;
            
            this._selectedEffectList.Clear();

            this.lblErosionAlert.Visible = false;
            StringBuilder effectCombo = new StringBuilder();

            foreach (DataGridViewRow gvr in this.dgvEffect.Rows)
            {
                CharacterDataSet.CharacterEffectsRow row = this.GetBindingRow(gvr);

                if (!Convert.IsDBNull(row["chk"]) && (bool)row["chk"])
                {
                    if (row.ErosionRateLimit > this.ErosionRateControl.ErosionRate)
                    {
                        this.lblErosionAlert.Visible = true;
                    }
                    else
                    {
                        DiceRoll rowDice = this.CreateRowDice(row);

                        this.Dice += rowDice.Dice;
                        this.Bonus += rowDice.Bonus;

                        if (rowDice.Critical != DiceRoll.NotCritical)
                        {
                            this.MinusCritical += 10 - rowDice.Critical;
                        }

                        if (row.CritUnderLimit != 0 && row.CritUnderLimit != DiceRoll.NotCritical && this.UnderCritical > row.CritUnderLimit)
                        {
                            this.UnderCritical = row.CritUnderLimit;
                        }

                        this.TotalErossionRate += row.ErosionRate;

                        //選択されているエフェクト名の追加
                        this._selectedEffectList.Add(row.EffectName);
                    }
                }
            }

            this.txtAddErosionRate.Text = this.TotalErossionRate.ToString("#,##0");

            //エフェクト分ダイス
            this.txtSelectedEffectDice.Text = DiceRoll.NewDiceRole(this.Dice, this.Bonus, 10 - this.MinusCritical, this.UnderCritical).ToString();

            this.OnCheckedChanged(new EventArgs());
        }
        #endregion

        #region dgvEffect_DataBindingComplete / データバインドされたとき
        /// <summary>
        /// データバインドされたとき
        /// </summary>
        private void dgvEffect_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
        {
            this.dgvEffect.AutoResizeColumns(DataGridViewAutoSizeColumnsMode.AllCells);

            using (Graphics g = this.CreateGraphics())
            {
                foreach (DataGridViewColumn column in this.dgvEffect.Columns)
                {
                    if (column is DataGridViewCheckBoxColumn)
                    {
                        //チェックボックスの時は、ちょっと独特の処理が必要
                        // > ヘッダの大きさであわせるか、チェックボックスのサイズで余裕をもってあわせる
                        const int checkBoxBaseWidth = 32;

                        int textWidth = checkBoxBaseWidth;

                        if (column.HeaderText.Length > 0)
                        {
                            Font columnFont = this.GetColumnFont(column);

                            if (columnFont == null)
                            {
                                throw new NotImplementedException("フォントが不明です。フレームワークに問題がある可能性があります。");
                            }

                            //その文字を描画するのに必要な幅が、あんまり正しくないので
                            textWidth = (int)g.MeasureString(column.HeaderText + "WW", columnFont).Width;

                            if (checkBoxBaseWidth > textWidth)
                            {
                                textWidth = checkBoxBaseWidth;
                            }
                        }
                        column.Width = textWidth;
                    }
                }
            }
        }
        #endregion

        #region GetColumnFont / カラムのフォントを取得する部分
        /// <summary>
        /// カラムのフォントを取得する部分
        /// </summary>
        /// <param name="column">フォントを取得するカラム</param>
        /// <returns>取得したフォント</returns>
        private Font GetColumnFont(DataGridViewColumn column)
        {
            if (column.DefaultCellStyle != null && column.DefaultCellStyle.Font != null)
            {
                return column.DefaultCellStyle.Font;
            }
            else if (this.dgvEffect.DefaultCellStyle != null && this.dgvEffect.DefaultCellStyle.Font != null)
            {
                return this.dgvEffect.DefaultCellStyle.Font;
            }

            return this.Font;
        }
        #endregion

        #region dgvEffect_KeyDown / 並び替え
        /// <summary>
        /// 並び替え
        /// </summary>
        private void dgvEffect_KeyDown(object sender, KeyEventArgs e)
        {
            if (this.dgvEffect.SelectedRows.Count > 0)
            {
                DataGridViewRow gvr = this.dgvEffect.SelectedRows[0];
                CharacterDataSet.CharacterEffectsRow row = this.GetBindingRow(gvr);

                DataView view = this.dsDataSet.CharacterEffects.DefaultView;
                
                int position = this.bndEffect.Position;

                if (e.KeyData == (Keys.Up | Keys.Control) && row.SerialNo != 0)
                {
                    //上へ
                    // > ひとつ前を自分にして、１つ減らす
                    view.RowFilter = String.Format("SerialNo < {0}", row.SerialNo);
                    view.Sort = "SerialNo DESC";

                    if (view.Count > 0)
                    {
                        CharacterDataSet.CharacterEffectsRow moveRow = (CharacterDataSet.CharacterEffectsRow)view[0].Row;
                        moveRow.SerialNo = 0;

                        int currentSerialNo = row.SerialNo;

                        row.SerialNo--;
                        moveRow.SerialNo = currentSerialNo;

                        this.bndEffect.Position = position - 1;
                        e.Handled = true;
                    }
                }
                else if (e.KeyData == (Keys.Down | Keys.Control) &&
                    Convert.ToInt32(this.dsDataSet.CharacterEffects.Compute("MAX(SerialNo)", null)) != row.SerialNo)
                {
                    //下へ
                    // > 一つ上を自分にして、自分の値をインクリメントする
                    view.RowFilter = String.Format("SerialNo > {0}", row.SerialNo);
                    view.Sort = "SerialNo";

                    if (view.Count > 0)
                    {
                        CharacterDataSet.CharacterEffectsRow moveRow = (CharacterDataSet.CharacterEffectsRow)view[0].Row;
                        moveRow.SerialNo = 0;

                        int currentSerialNo = row.SerialNo;
                        
                        row.SerialNo++;
                        moveRow.SerialNo = currentSerialNo;

                        this.bndEffect.Position = position + 1;
                        e.Handled = true;
                    }
                }
                
            }
        }
        #endregion
    }
}
