﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Reflection;

namespace DoubleX3rd
{
    /// <summary>
    /// Enumで、ちょっとした仕掛けをするためのもの。C#3.0な環境なら、拡張メソッドでいいんだけどね
    /// </summary>
    [AttributeUsage(AttributeTargets.Field, AllowMultiple = false, Inherited = true)]
    public sealed class EnumStringAttribute : Attribute
    {
        /// <summary>
        /// LabeledEnumAttribute クラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="label">ラベルとして使用する文字列。</param>
        public EnumStringAttribute(string label)
        {
            this.Label = label;
        }

        /// <summary>
        /// enumのラベルを取得します。
        /// </summary>
        public string Label { get; private set; }

        /// <summary>
        /// 属性で指定されたラベル文字列を取得します。
        /// </summary>
        /// <param name="value">取得するEnumの値</param>
        /// <returns>ラベル文字列</returns>
        public static string GetLabel(Enum value)
        {
            Type enumType = value.GetType();

            string name = Enum.GetName(enumType, value);

            EnumStringAttribute[] attrs = (EnumStringAttribute[])enumType.GetField(name).GetCustomAttributes(typeof(EnumStringAttribute), true);

            if (attrs.Length > 0)
            {
                return attrs[0].Label;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// 指定されたEnumから、属性のLabel値と列挙の値を取得して、列挙の値をKeyとした、KeyValuePair のリストを取得します。
        /// </summary>
        /// <param name="enumType">EnumStringAttribute でマークされたEnumの型</param>
        /// <returns>EnumStringAttribute の Value を Key とした、KeyValuePair のリスト</returns>
        public static KeyValuePair<int, string>[] GetEnumValueLabelArray(Type enumType)
        {
            return GetEnumValueLabelArray(enumType, false);
        }

        /// <summary>
        /// 指定されたEnumから、属性のLabel値とValue値を取得して、ValueをKeyとした、KeyValuePair のリストを取得します。
        /// </summary>
        /// <param name="enumType">EnumStringAttribute でマークされたEnumの型</param>
        /// <param name="codeBeforeLabel">Value に設定される Label の前に、コードを挿入するかを指定します。</param>
        /// <returns>EnumStringAttribute の列挙の値を Key とした、KeyValuePair のリスト</returns>
        /// <note>手抜きなので、暗黙的に列挙の値はintね。</note>
        public static KeyValuePair<int, string>[] GetEnumValueLabelArray(Type enumType, bool codeBeforeLabel)
        {
            string[] names = Enum.GetNames(enumType);

            List<KeyValuePair<int, string>> result = new List<KeyValuePair<int, string>>();

            foreach (string name in names)
            {
                FieldInfo fieldInfo = enumType.GetField(name);
                EnumStringAttribute[] attrs = (EnumStringAttribute[])fieldInfo.GetCustomAttributes(typeof(EnumStringAttribute), true);

                if (attrs.Length == 1)
                {
                    int enumValue = (int)Enum.Parse(enumType, name);

                    result.Add(new KeyValuePair<int, string>(enumValue, (codeBeforeLabel) ? String.Format(CultureInfo.InstalledUICulture, "{0}:{1}",  enumValue, attrs[0].Label): attrs[0].Label));
                }
                else
                {
                    throw new ArgumentException();
                }
            }

            return result.ToArray();
        }
    }
}
