﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// スキル用のコントロール。たぶん面倒なんで、いろいろ使い回す
    /// </summary>
    [DefaultEvent("CheckedChanged")]
    public partial class SkillListControl : UserControl
    {
        #region constructor
        /// <summary>
        /// SkillListControl の新しいインスタンスを生成します。
        /// </summary>
        public SkillListControl()
        {
            InitializeComponent();
        }
        #endregion

        #region OnLoad
        /// <summary>
        /// OnLoad
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            //未設定を追加しておく
            if (!this.DesignMode)
            {
                this.lstSkill.Items.Add(new SkillListItem("未設定", 0));
            }
        }
        #endregion

        #region btnAdd_Click / 追加クリック
        /// <summary>
        /// 追加クリック
        /// </summary>
        private void btnAdd_Click(object sender, EventArgs e)
        {
            using (AddSkill frm = new AddSkill())
            {
                if (frm.ShowDialog(this) == DialogResult.OK)
                {
                    this.AddSkill(frm.SkillName, frm.SkillLv);
                }
            }
        }
        #endregion

        #region class SkillListItem / スキルアイテム用のデータクラス
        /// <summary>
        /// スキルアイテム用のデータクラス
        /// </summary>
        public class SkillListItem
        {
            /// <summary>
            /// SkillListItem の新しいインスタンスを生成します。
            /// </summary>
            /// <param name="skillName">スキル名</param>
            /// <param name="skillLevel">スキルレベル</param>
            public SkillListItem(string skillName, int skillLevel)
            {
                this.SkillName = skillName;
                this.SkillLevel = skillLevel;
            }

            /// <summary>
            /// スキル名を取得します。
            /// </summary>
            public string SkillName { get; private set; }

            /// <summary>
            /// スキルレベルを取得します。
            /// </summary>
            public int SkillLevel { get; private set; }

            public override string ToString()
            {
                return String.Format("{0}:{1}Lv", this.SkillName, this.SkillLevel);
            }

            /// <summary>
            /// スキル名で、アイテムが同一かどうかをチェックします。
            /// </summary>
            /// <param name="obj">比較するオブジェクト</param>
            /// <returns>同一のスキル名の時 <c>true</c> 。</returns>
            public override bool Equals(object obj)
            {
                SkillListItem source = obj as SkillListItem;

                if (source != null)
                {
                    return (this.SkillName == source.SkillName);
                }

                return base.Equals(obj);
            }

            public override int GetHashCode()
            {
                return this.SkillName.GetHashCode();
            }
        }
        #endregion

        #region Clear / スキルをクリアする
        /// <summary>
        /// スキルをクリアする
        /// </summary>
        public void Clear()
        {
            this.lstSkill.Items.Clear();
        }
        #endregion

        #region AddSkill / スキルを追加する
        /// <summary>
        /// スキルを追加する
        /// </summary>
        /// <param name="skillName">スキル名</param>
        /// <param name="skillLevel">スキルレベル</param>
        public void AddSkill(string skillName, int skillLevel)
        {
            List<SkillListItem> skillList = new List<SkillListItem>(this.lstSkill.Items.Count);

            for (int i = 0; i < this.lstSkill.Items.Count; i++)
            {
                skillList.Add((SkillListItem)this.lstSkill.Items[i]);
            }

            SkillListItem newSkillItem = new SkillListItem(skillName, skillLevel);

            if (skillList.Contains(newSkillItem))
            {
                MessageBox.Show("すでに追加されています。");
            }
            else
            {
                this.lstSkill.Items.Add(newSkillItem);
            }
        }
        #endregion

        #region GetAllSkillListItem / すべてのスキルリストアイテムを取得する
        /// <summary>
        /// すべてのスキルリストアイテムを取得する
        /// </summary>
        /// <returns>取得したスキルリストアイテム</returns>
        public SkillListItem[] GetAllSkillListItem()
        {
            List<SkillListItem> result = new List<SkillListItem>(this.lstSkill.Items.Count);

            foreach (SkillListItem item in this.lstSkill.Items)
            {
                result.Add(item);
            }

            return result.ToArray();
        }
        #endregion

        #region btnRemove_Click / 削除クリック
        /// <summary>
        /// 削除クリック
        /// </summary>
        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (this.lstSkill.SelectedIndex != -1)
            {
                if (MessageBox.Show(this, "選択された項目を削除します。", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    this.lstSkill.Items.RemoveAt(this.lstSkill.SelectedIndex);
                }
            }
        }
        #endregion

        #region lstSkill_ItemCheck / 一個しか選べない
        /// <summary>
        /// 一個しか選べない
        /// </summary>
        private void lstSkill_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.CurrentValue == CheckState.Unchecked)
            {
                for (int i = 0; i < this.lstSkill.Items.Count; i++)
                {
                    if (this.lstSkill.GetItemCheckState(i) == CheckState.Checked &&
                        i != e.Index)
                    {
                        this.lstSkill.SetItemChecked(i, false);
                    }
                }
            }

            if (e.NewValue == CheckState.Checked)
            {
                OnCheckedChanged(e);
            }
        }
        #endregion

        #region event CheckedChanged / チェックされた時に発生します。
        /// <summary>
        /// チェックされた時に発生します。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("チェックされた時に発生します。")]
        public event ItemCheckEventHandler CheckedChanged;

        /// <summary>
        /// <see cref="CheckedChanged"/> イベントを発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        protected virtual void OnCheckedChanged(ItemCheckEventArgs e)
        {
            if (this.CheckedChanged != null)
            {
                this.CheckedChanged(this, e);
            }
        }
        #endregion

        #region GetCheckedSkillLevel / 選択されている項目のスキルレベルを取得する
        /// <summary>
        /// 選択されている項目のスキルレベルを取得する
        /// </summary>
        /// <returns>選択されているレベル</returns>
        public int GetCheckedSkillLevel()
        {
            if (this.lstSkill.CheckedIndices.Count > 0)
            {
                return this.GetItemSkillLevel(this.lstSkill.CheckedIndices[0]);
            }
            return 0;
        }
        #endregion

        #region GetItemSkillLevel / 指定されたindexのスキルレベルを取得します。
        /// <summary>
        /// 指定されたindexのスキルレベルを取得します。
        /// </summary>
        /// <param name="index">取得するindex</param>
        /// <returns>取得したレベル</returns>
        public int GetItemSkillLevel(int index)
        {
            SkillListItem item = (SkillListItem)this.lstSkill.Items[index];
            return item.SkillLevel;
        }
        #endregion

        #region IsChecked / チェックされているか
        /// <summary>
        /// チェックされているか
        /// </summary>
        [Browsable(false)]
        public bool IsChecked
        {
            get
            {
                for (int i = 0; i < this.lstSkill.Items.Count; i++)
                {
                    if (this.lstSkill.GetItemCheckState(i) == CheckState.Checked)
                    {
                        return true;
                    }
                }

                return false;
            }
        }
        #endregion

        #region ClearCheck / チェックを全部クリアする
        /// <summary>
        /// チェックを全部クリアする
        /// </summary>
        public void ClearCheck()
        {
            this.lstSkill.ClearSelected();

            for (int i = 0; i < this.lstSkill.Items.Count; i++)
            {
                if (this.lstSkill.GetItemCheckState(i) == CheckState.Checked)
                {
                    this.lstSkill.SetItemChecked(i, false);
                }
            }
        }
        #endregion

        #region lstSkill_KeyDown / キーで上下で並び順の変更
        /// <summary>
        /// キーで上下で並び順の変更
        /// </summary>
        private void lstSkill_KeyDown(object sender, KeyEventArgs e)
        {
            if (this.lstSkill.SelectedIndices.Count > 0)
            {
                int selectedIndex = this.lstSkill.SelectedIndices[0];

                if (e.KeyData == (Keys.Up | Keys.Control) && selectedIndex != 0)
                {
                    //上へ
                    SkillListItem item = (SkillListItem)this.lstSkill.Items[selectedIndex];
                    this.lstSkill.Items.RemoveAt(selectedIndex);
                    this.lstSkill.Items.Insert(selectedIndex - 1, item);

                    this.lstSkill.SetSelected(selectedIndex - 1, true);
                    e.Handled = true;
                }
                else if (e.KeyData == (Keys.Down | Keys.Control) && this.lstSkill.Items.Count > selectedIndex + 1)
                {
                    //下へ
                    SkillListItem item = (SkillListItem)this.lstSkill.Items[selectedIndex];
                    this.lstSkill.Items.RemoveAt(selectedIndex);
                    this.lstSkill.Items.Insert(selectedIndex + 1, item);

                    this.lstSkill.SetSelected(selectedIndex + 1, true);
                    e.Handled = true;
                }
            }
        }
        #endregion
    }
}
